/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GenerateDialog.cpp,v 2.3 1999/10/16 01:43:36 jeff Exp $

*******************************************************************************/


#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qpushbt.h>
#include <qmsgbox.h>
#include <qlabel.h>

#include "GenerateDialog.h"

#define Inherited QDialog

GenerateDialog::GenerateDialog
(
    const QString&	fileName,
    const QString&	dlgClassName,
    const QString&	classHeader,
    const QString&	classSource,
    const QString&	dataHeader,
    const QString&	dataSource,
    QWidget*		parent,
    const char*		name
)
    :
    Inherited( parent, name, TRUE ),
    itsSourceCheckbox( tr( "Generate dialog class files" ), this ),
    itsSourceGB( tr( "Filenames..." ), this ),
    itsClassHeaderLabel( tr( "Header:" ), this ),
    itsClassSourceLabel( tr( "Source:" ), this ),
    itsClassHeaderEdit( this ),
    itsClassSourceEdit( this ),
    itsDataCheckbox( tr( "Generate dialog data class files" ), this ),
    itsDataGB( tr( "Filenames..." ), this ),
    itsDataHeaderLabel( tr( "Header:" ), this ),
    itsDataSourceLabel( tr( "Source:" ), this ),
    itsDataHeaderEdit( this ), 
    itsDataSourceEdit( this )
{
    setCaption( tr( "Generate Source" ) );

    QLabel* label = new QLabel( tr( "Dialog file: %1" ).arg( fileName ), this );
    label->adjustSize();
    label->move( 20, 10 );
    
    label = new QLabel(tr( "Dialog class: %1" ).arg( dlgClassName ), this );
    label->adjustSize();
    label->move( 20, 25 );

    itsSourceCheckbox.setGeometry( 20, 40, 190, 30 );
    itsSourceGB.setGeometry( 40, 70, 330, 50 );
    itsClassHeaderLabel.setGeometry( 50, 85, 50, 30 );
    itsClassSourceLabel.setGeometry( 215, 85, 40, 30 );
    itsClassHeaderEdit.setGeometry( 100, 90, 100, 20 );
    itsClassSourceEdit.setGeometry( 260, 90, 100, 20 );
    itsDataCheckbox.setGeometry( 20, 125, 220, 30 );
    itsDataGB.setGeometry( 40, 155, 330, 50 );
    itsDataHeaderLabel.setGeometry( 50, 170, 50, 30 );
    itsDataSourceLabel.setGeometry( 215, 170, 40, 30 );
    itsDataHeaderEdit.setGeometry( 100, 175, 100, 20 );
    itsDataSourceEdit.setGeometry( 260, 175, 100, 20 );

    itsClassHeaderEdit.setText( classHeader );
    itsClassSourceEdit.setText( classSource );
    itsDataHeaderEdit.setText( dataHeader );
    itsDataSourceEdit.setText( dataSource );

    QFileInfo fileInfo( fileName );
    QDir fileDir = fileInfo.dir();

    bool checked = 
	(classHeader.isEmpty() || 
         !QFile::exists( QFileInfo( fileDir, classHeader ).filePath() )) &&
	(classSource.isEmpty() || 
         !QFile::exists( QFileInfo( fileDir, classSource ).filePath() ));

    itsSourceCheckbox.setChecked( checked );
    connect( &itsSourceCheckbox, SIGNAL(toggled(bool)), 
             SLOT(GetSourceNames(bool)) );
    GetSourceNames( checked );

    itsDataCheckbox.setChecked( TRUE );
    connect( &itsDataCheckbox, SIGNAL(toggled(bool)), 
             SLOT(GetDataNames(bool)) );
    GetDataNames( TRUE );
    
    QPushButton* button = new QPushButton( tr( "Generate" ), this );
    button->setGeometry( 20, 215, 100, 30 );
    connect( button, SIGNAL(clicked()), SLOT(EndDialog()) );
    
    button = new QPushButton( tr( "Cancel" ), this );
    button->setGeometry( 280, 215, 100, 30 );
    connect( button, SIGNAL(clicked()), SLOT(reject()) );

    adjustSize();
}

bool GenerateDialog::GenerateSource( QString &header, QString &source )
{
  bool ret = itsSourceCheckbox.isChecked();
  if( ret )
  {
    header = itsClassHeaderEdit.text();
    source = itsClassSourceEdit.text();
  }
  return ret; 
}

bool GenerateDialog::GenerateData( QString &header, QString &source )
{
  bool ret = itsDataCheckbox.isChecked();
  if( ret )
  {
    header = itsDataHeaderEdit.text();
    source = itsDataSourceEdit.text();
  }
  return ret; 
}

/* Slot called by itsSourceCheckbox whenever its Status changed
 *
 * Display the LineEdits for the Source names when Checkbox is on
 */
void GenerateDialog::GetSourceNames( bool on )
{
    itsSourceGB.setEnabled( on );
    itsClassHeaderLabel.setEnabled( on );
    itsClassSourceLabel.setEnabled( on );
    itsClassHeaderEdit.setEnabled( on );
    itsClassSourceEdit.setEnabled( on );
}

/* Slot called by itsDataCheckbox whenever its Status changed
 *
 * Display the LineEdits for the Data names when Checkbox is on
 */
void GenerateDialog::GetDataNames( bool on )
{
    itsDataGB.setEnabled( on );
    itsDataHeaderLabel.setEnabled( on );
    itsDataSourceLabel.setEnabled( on );
    itsDataHeaderEdit.setEnabled( on );
    itsDataSourceEdit.setEnabled( on );
}

/* Slot called by the OK-Button
 *
 * Check wether all LineEdits which are visible have a text
 * If so, end the dialog with accept, otherwise bring up a messagebox
 * which explains the horrible error which was made by the user...
 */
void GenerateDialog::EndDialog()
{
  QString sC, hC;
  QString sD, hD;

  int errorCode = 0;

  if( itsSourceCheckbox.isChecked() )
  {
    hC = itsClassHeaderEdit.text();
    sC = itsClassSourceEdit.text();
  }

  if( itsDataCheckbox.isChecked() )
  {
    hD = itsDataHeaderEdit.text();
    sD = itsDataSourceEdit.text();
  }

  if( itsSourceCheckbox.isChecked() )
  {
    if( sC == "" )
      errorCode |= 1;
    if( hC == "" )
      errorCode |= 2;
  }
  if( itsDataCheckbox.isChecked() )
  {
    if( sD == "" )
      errorCode |= 4;
    if( hD == "" )
      errorCode |= 8;
  }

  // Some of the following error conditions are, mh, absurd
  QString errorText;
  switch( errorCode )
  {
  case 0:
    accept();
    break;
  case 1:
    errorText = tr( "No file for the class header specified" );
    break;
  case 2:
    errorText = tr( "No file for the class source specified" );
    break;
  case 3:
    errorText = tr( "No files for the class specified" );
    break;
  case 4:
    errorText = tr( "No file for the data header specified" );
    break;
  case 5:
    errorText = tr( "No file for the headers specified" );
    break;
  case 6:
    errorText = tr( "No file for the class source and the data header specified" );
    break;
  case 7:
    errorText = tr( "No file for the class and the data header specified" );
    break;
  case 8:
    errorText = tr( "No file for the data source specified" );
    break;
  case 9:
    errorText = tr( "No file for the class header and data source specified" );
    break;
  case 10:
    errorText = tr( "No files for the sources specified" );
    break;
  case 11:
    errorText = tr( "No files for the class and the data source specified" );
    break;
  case 12:
    errorText = tr( "No files for the data specified" );
    break;
  case 13:
    errorText = tr( "No files for the class header and the data specified" );
    break;
  case 14:
    errorText = tr( "No files for the class source and the data specified" );
    break;
  case 15:
    errorText = tr( "No files specified" );
    break;
  }

  if( errorCode )
  {
      QMessageBox::message( tr( "Error..." ),
			    errorText,
			    tr( "Try again" ),
			    this );
  }
}












