/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GuiTools.h,v 2.9 2000/04/30 17:09:37 jeff Exp $

*******************************************************************************/


#ifndef GuiTools_included
#define GuiTools_included

#include <qcombo.h>
#include <qdrawutil.h>
#include <qfont.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qstring.h>
#include <qstrlist.h>
#include <qwidget.h>

class GuiTools
{
public:

    GuiTools();
    ~GuiTools();
    // the ctor and dtor are declared but never implemented


    //
    // QApplication
    //

    static const QString platinumStyle;
    static const QString cdeStyle;
    static const QString motifStyle;
    static const QString windowsStyle;

    static void SetStyle( const QString& style );
    // Set the application's style to the one specified.

    //
    // QWidget
    //

    static void SetVisible( QWidget*	widget,
		       bool	visible );
    // show or hide the widget

    static QRect GetGlobalGeometry
    (
	const QWidget*	widget
    );
    // get the geometry rect of the widget in world coordinates

    static void ChangeColor( 
        QWidget*        widget,
        const QColor&   color
        );
    // Change the color of the given widget to the specified color

    //
    // QListBox
    //

    static void TransferListBox( const QListBox*	source,
				 QListBox*		dest );
    // transfer list from the source list box to the destination


    //
    // QComboBox
    //

    static void TransferListFromCombo( const QComboBox*	source,
				       QListBox*	dest );
    // transfer list from the source combo box to the list box

    static void TransferListToCombo( QComboBox*		dest,
				     const QListBox*	source );
    // transfer list from the source list box to the combo box


    //
    // QDialog
    //
    
    static const QStrList& GetQDialogSlots( void );
    // Return a list of slots that are defined by a QDialog


    //
    // QFont
    //

    static QString QFontStyleHintStr( QFont::StyleHint	hint );
    // return a string value for the enumeration

    static QFont::StyleHint QFontStyleHint( QString	str );
    // return the style hint for the string, default to Helvetica

    static QString QFontWeightStr( int	weight );
    // return a string value for the wieght

    static QString QFontCharSetStr( QFont::CharSet	set );
    // return a string value for the set

    static QFont::CharSet QFontCharSet( QString		set );
    // return the char set for the string, default to Latin1


    //
    // QCursor
    //

    static QString QCursorStr( const QCursor&   cursor );
    static const QCursor& QCursorVal( const QString&    str );
    

    //
    // QStrList
    //

    static void QStrListAppendAll( QStrList&		dest,
				   const QStrList&	source );
    // append all the items in the source to the destination

    
    //
    // QListView
    //

    static int numSiblings( const QListViewItem* theItem );
    // Count how many siblings this item has.  This item should be the 
    // first sibling.

    //
    // Misc
    //

    static QString ArrowTypeStr(
	Qt::ArrowType	theArrowType
	);
    // Return a string for the arrow type

    static Qt::ArrowType ArrowTypeVal(
	const QString&	theArrowTypeStr
	);
    // Return an ArrowType or the arrow type

private:

    GuiTools( const GuiTools& );
    const GuiTools& operator = ( const GuiTools& );
    
    // no one should ever declare a GuiTools, just use static members

    static QStrList	itsQDialogSlots;
    // slots defined by a QDialog
};


#endif // GuiTools_included
