/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: BoxStretch.cpp,v 2.5 2000/04/29 20:15:46 klaus Exp $

*******************************************************************************/


#include "layout/BoxStretch.h"
#include "layout/BoxStretchPropPage.h"
#include "layout/LayoutHelper.h"

#include "xml/QtArchGenerator.h"

#define Inherited LayoutItem

BoxStretch::BoxStretch(
    LayoutItem*	theParent
    )
    :
    Inherited( LayoutItem::LI_BoxStretch, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( BoxStretch_bmp ) ),
    myStretch( 1 )
{
}


BoxStretch::BoxStretch(
    QListView*	theParent
    )
    :
    Inherited( LayoutItem::LI_BoxStretch, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( BoxStretch_bmp ) ),
    myStretch( 1 )
{
}


BoxStretch::~BoxStretch()
{
}


LayoutItemPropPage* BoxStretch::GetPropertyPage( QWidget* theParent,
						 bool	/*isTopLevel*/,
						 const WidgetLayout&	/*theWidgetLayout*/ )
{
    return new BoxStretchPropPage( this, theParent );
}


void BoxStretch::AddToLayout( QBoxLayout*	theBoxLayout )
{
    theBoxLayout->addStretch( myStretch );
}


void BoxStretch::SaveContents( QtArchGenerator &generator ) const
{
  generator.WriteStartBlock( Type2String() );

  generator.WriteXMLValue( "Stretch", myStretch );

  generator.WriteEndBlock();
}


bool BoxStretch::RestoreKeyValue( const QString&	theKey,
				  const QString&	theValue,
				  const DlgWidget*	/*theDlgWidget*/ )
{
    bool rc = TRUE;

    if( theKey == "Stretch" )
    {
	myStretch = theValue.toInt();
    }
    else
    {
	rc = FALSE;
    }
    
    return rc;
}

void BoxStretch::Restore( const XMLValue &value, LayoutHelper */*helper*/ )
{
  // Sanity check
  if( value.GetTypeString() != Type2String() )
    return;

  const XMLValueMap map = value.AsNested();

  myStretch = map["Stretch"].AsInt();
}



void BoxStretch::GenerateSourceInBox( QTextStream&	theStream,
				      const QString&	theLayoutVarName,
				      const QString&	/*theLayoutId*/,
				      const QString&	/*theLayoutParentWidget*/ ) const
{
    theStream << '\t' << theLayoutVarName << "->addStretch( " <<
	myStretch << " );\n";
}


QString BoxStretch::GetTreeLabel()
{
    static QString treeLabel = QObject::tr( "Box Stretch" );
    return treeLabel;
}
