/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutWnd.cpp,v 2.5 2000/01/17 01:20:39 jeff Exp $

*******************************************************************************/


#include <qiconset.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>

#include "layout/LayoutWnd.h"

#include "layout/LayoutHelper.h"
#include "layout/NoLayout.h"
#include "layout/BoxLayout.h"
#include "layout/BoxSpacing.h"
#include "layout/BoxStretch.h"
#include "layout/GridLayout.h"
#include "layout/GridRow.h"
#include "layout/GridSpacer.h"
#include "layout/LayoutWidget.h"

#include "bmp/PixmapFactory.h"

#define Inherited QMainWindow

LayoutWnd::LayoutWnd( LayoutHelper&	theLayoutHelper,
		      QListView*&	layoutTree,
		      QWidget*		parent,
		      const char*	name )
    :
    Inherited( parent, name ),
    myLayoutHelper( theLayoutHelper ),
    mySplitter( QSplitter::Horizontal, this, "Panner" ),
    myLayoutTree( NULL ),
    myPropertyFrame( 0 ),
    myRootTreeItem( NULL ),
    myPreviousItem( NULL ),
    myKbdAccel( this )
{
    setIcon( PixmapFactory::GetPixmap( QtArch_bmp ) );

    // Initialize menu bar
    QPopupMenu* layoutMenu = new QPopupMenu;
    layoutMenu->insertItem( tr( "&Close" ), this, SLOT(Close()), Key_C + CTRL );

    QPopupMenu* editMenu = new QPopupMenu;
    editMenu->insertItem( tr( "&Delete" ), this, SLOT(EditDeleteItem()), 
			  Key_Delete );
    
    QPopupMenu* insertMenu = new QPopupMenu;
    myInsertBoxLayoutMenuId = 
	insertMenu->insertItem( PixmapFactory::GetPixmap( BoxLayout_bmp ),
				tr( "&Box Layout" ), 
				this, 
				SLOT(InsertBoxLayout()),
				Key_B + CTRL );
    myInsertBoxSpacingMenuId =
	insertMenu->insertItem( PixmapFactory::GetPixmap( BoxSpacing_bmp ),
				tr( "S&pacing" ), 
				this, 
				SLOT(InsertBoxSpacing()),
				Key_P + CTRL );
    myInsertBoxStretchMenuId = 
	insertMenu->insertItem( PixmapFactory::GetPixmap( BoxStretch_bmp ),
				tr( "S&tretch" ), 
				this, 
				SLOT(InsertBoxStretch()),
				Key_T + CTRL );
    insertMenu->insertSeparator();
    myInsertGridLayoutMenuId =
	insertMenu->insertItem( PixmapFactory::GetPixmap( GridLayout_bmp ),
				tr( "&Grid Layout" ), 
				this, 
				SLOT(InsertGridLayout()),
				Key_G + CTRL );
    myInsertGridRowMenuId = 
	insertMenu->insertItem( PixmapFactory::GetPixmap( GridRow_bmp ),
				tr( "Grid &Row" ), 
				this, 
				SLOT(InsertGridRow()),
				Key_R + CTRL );
    myInsertGridSpacerMenuId =
	insertMenu->insertItem( PixmapFactory::GetPixmap( GridSpacer_bmp ),
				tr( "Grid &Spacer" ), 
				this, 
				SLOT(InsertGridSpacer()),
				Key_S + CTRL );
    insertMenu->insertSeparator();
    myInsertLayoutWidgetMenuId =
	insertMenu->insertItem( PixmapFactory::GetPixmap( LCDNumber_bmp ),
				tr( "&Widget" ), 
				this, 
				SLOT(InsertLayoutWidget()),
				Key_W + CTRL );

    QMenuBar& menu = *menuBar();
    menu.insertItem( tr( "&Layout" ), layoutMenu );
    menu.insertItem( tr( "&Edit" ), editMenu );
    menu.insertItem( tr( "&Insert" ), insertMenu );

    // Initialize toolbar
    QToolBar* toolBar = new QToolBar( this );

    myInsertBoxLayoutTbId = 
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( BoxLayout_bmp ), 
				   QIconSet::Small ),
			 tr( "Insert a QBoxLayout (Ctrl + B)" ),
			 tr( "Insert a QBoxLayout (Ctrl + B)" ),
			 this,
			 SLOT(InsertBoxLayout()),
			 toolBar );

    myInsertBoxSpacingTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( BoxSpacing_bmp ),
				   QIconSet::Small ),
			 tr( "Insert Spacing (Ctrl + P)" ),
			 tr( "Insert Spacing (Ctrl + P)" ),
			 this,
			 SLOT(InsertBoxSpacing()),
			 toolBar );

    myInsertBoxStretchTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( BoxStretch_bmp ),
				   QIconSet::Small ),
			 tr( "Insert Stretch (Ctrl + T)" ),
			 tr( "Insert Stretch (Ctrl + T)" ),
			 this,
			 SLOT(InsertBoxStretch()),
			 toolBar );

    toolBar->addSeparator();
    
    myInsertGridLayoutTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( GridLayout_bmp ),
				   QIconSet::Small ),
			 tr( "Insert a QGridLayout (Ctrl + G)" ),
			 tr( "Insert a QGridLayout (Ctrl + G)" ),
			 this,
			 SLOT(InsertGridLayout()),
			 toolBar );
    
    myInsertGridRowTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( GridRow_bmp ),
				   QIconSet::Small ),
			 tr( "Insert a Row in the Grid (Ctrl + R)" ),
			 tr( "Insert a Row in the Grid (Ctrl + R)" ),
			 this,
			 SLOT(InsertGridRow()),
			 toolBar );
    
    myInsertGridSpacerTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( GridSpacer_bmp ),
				   QIconSet::Small ),
			 tr( "Insert a Blank Spacer in the Grid (Ctrl + S)" ),
			 tr( "Insert a Blank Spacer in the Grid (Ctrl + S)" ),
			 this,
			 SLOT(InsertGridSpacer()),
			 toolBar );

    toolBar->addSeparator();
    
    myInsertLayoutWidgetTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( LCDNumber_bmp ),
				   QIconSet::Small ),
			 tr( "Insert a Widget into the layout (Ctrl + W)" ),
			 tr( "Insert a Widget into the layout (Ctrl + W)" ),
			 this,
			 SLOT(InsertLayoutWidget()),
			 toolBar );
    
    toolBar->addSeparator();
    
    myMoveUpTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( UpArrow_bmp ),
				   QIconSet::Small ),
			 tr( "Move Selected Item Up (Ctrl + Up)" ),
			 tr( "Move Selected Item Up (Ctrl + Up)" ),
			 this,
			 SLOT(MoveItemUp()),
			 toolBar );

    myMoveDownTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( DownArrow_bmp ),
				   QIconSet::Small ),
			 tr( "Move Selected Item Down (Ctrl + Down)" ),
			 tr( "Move Selected Item Down (Ctrl + Down)" ),
			 this,
			 SLOT(MoveItemDown()),
			 toolBar );

    myMoveLeftTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( LeftArrow_bmp ),
				   QIconSet::Small ),
			 tr( "Move Selected Item Left (Ctrl + Left)" ),
			 tr( "Move Selected Item Left (Ctrl + Left)" ),
			 this,
			 SLOT(MoveItemLeft()),
			 toolBar );

    myMoveRightTbId =
	new QToolButton( QIconSet( PixmapFactory::GetPixmap( RightArrow_bmp ),
				   QIconSet::Small ),
			 tr( "Move Selected Item Right (Ctrl + Right)" ),
			 tr( "Move Selected Item Right (Ctrl + Right)" ),
			 this,
			 SLOT(MoveItemRight()),
			 toolBar );


    // Initialize layout tree
    layoutTree = new QListView( &mySplitter );
    layoutTree->addColumn( tr( "Layout Items" ) );
    layoutTree->setSorting( -1 );
    myLayoutTree = layoutTree;
    layoutTree->resize( 300, layoutTree->height() );

    connect( myLayoutTree, SIGNAL(currentChanged(QListViewItem*)), 
	     SLOT(TreeSelChanged(QListViewItem*)) );

    connect( myLayoutTree, SIGNAL(doubleClicked(QListViewItem*)),
             SLOT(TreeDoubleClicked(QListViewItem*)) );

    connect( myLayoutTree, SIGNAL(returnPressed(QListViewItem*)),
             SLOT(TreeDoubleClicked(QListViewItem*)) );

    // Initialize property dialog
    myPropertyFrame = new LayoutPropFrame( &mySplitter );
    connect( myPropertyFrame, 
	     SIGNAL(ApplyProperties()), 
	     SLOT(ForceUpdateScreen()) );

    connect( myPropertyFrame,
	     SIGNAL(ApplyProperties()),
	     SIGNAL(ContentsChanged()) );


    InsertRootLayout( LayoutItem::LI_NoLayout );


    // Initialize window layout
    setCentralWidget( &mySplitter );
    
    resize( 600,400 );

    // Initialize keyboard accelerators
    myMoveUpAccelId = myKbdAccel.insertItem( Key_Up + CTRL );
    myMoveDownAccelId = myKbdAccel.insertItem( Key_Down + CTRL );
    myMoveLeftAccelId = myKbdAccel.insertItem( Key_Left + CTRL );
    myMoveRightAccelId = myKbdAccel.insertItem( Key_Right + CTRL );

    myKbdAccel.connectItem( myMoveUpAccelId, this, SLOT(MoveItemUp()) );
    myKbdAccel.connectItem( myMoveDownAccelId, this, SLOT(MoveItemDown()) );
    myKbdAccel.connectItem( myMoveLeftAccelId, this, SLOT(MoveItemLeft()) );
    myKbdAccel.connectItem( myMoveRightAccelId, this, SLOT(MoveItemRight()) );
    
    UpdateScreen();
}


LayoutWnd::~LayoutWnd()
{
    delete myLayoutTree;
}


void LayoutWnd::closeEvent(
    QCloseEvent*	theCloseEvent
    )
{
    Inherited::closeEvent( theCloseEvent );
    
    emit Closed();
}


void LayoutWnd::EditDeleteItem()
{
    myPropertyFrame->RemovePropertyPage();

    QListViewItem* current = myLayoutTree->currentItem();
    if( current == myRootTreeItem )
    {
	InsertRootLayout( LayoutItem::LI_NoLayout );
    }
    else
    {
	delete current;
    }

    emit ContentsChanged();

    UpdateScreen( TRUE );
}


void LayoutWnd::InsertBoxLayout()
{
    if( NoLayoutDefined() )
    {
	InsertRootLayout( LayoutItem::LI_BoxLayout );
    }
    else
    {
	InsertLayoutItem( LayoutItem::LI_BoxLayout );
    }

    emit ContentsChanged();
}


void LayoutWnd::InsertBoxSpacing()
{
    InsertLayoutItem( LayoutItem::LI_BoxSpacing );

    emit ContentsChanged();
}


void LayoutWnd::InsertBoxStretch()
{
    InsertLayoutItem( LayoutItem::LI_BoxStretch );

    emit ContentsChanged();
}


void LayoutWnd::InsertGridLayout()
{
    if( NoLayoutDefined() )
    {
	InsertRootLayout( LayoutItem::LI_GridLayout );
    }
    else
    {
	InsertLayoutItem( LayoutItem::LI_GridLayout );
    }

    emit ContentsChanged();
}


void LayoutWnd::InsertGridRow()
{
    InsertLayoutItem( LayoutItem::LI_GridRow );

    emit ContentsChanged();
}


void LayoutWnd::InsertGridSpacer()
{
    InsertLayoutItem( LayoutItem::LI_GridSpacer );
    
    emit ContentsChanged();
}


void LayoutWnd::InsertLayoutWidget()
{
    InsertLayoutItem( LayoutItem::LI_LayoutWidget );

    emit ContentsChanged();
}


void LayoutWnd::MoveItemUp()
{
    QListViewItem* currItem = myLayoutTree->currentItem();
    if( currItem )
    {
	QListViewItem* parent = currItem->parent();
	if( parent )
	{
	    myLayoutTree->setSelected( currItem, FALSE );

	    // Put the items up to, but not including the current item into
	    // the children list.
	    QList<QListViewItem> children;
	    QListViewItem* child = parent->firstChild();
	    QListViewItem* prevChild = 0;
            while( child != currItem )
	    {
                // Have to update child before removing it.
                prevChild = child;
                child = child->nextSibling();

		parent->removeItem( prevChild );
		
		// Reverse the order of the children
		children.insert( 0, prevChild );
	    }

	    parent->removeItem( currItem );

	    // Remove the first element which is the child previous to the
	    // current.
	    children.removeFirst();
	    
	    // Insert the previous, current, then the list of children
	    parent->insertItem( prevChild );

	    parent->insertItem( currItem );
	    
	    QListIterator<QListViewItem> childIter( children );
	    while( (child = childIter()) )
	    {
		parent->insertItem( child );
	    }

	    myLayoutTree->setSelected( currItem, TRUE );

	    emit ContentsChanged();
	    ForceUpdateScreen();
	}
    }
}


void LayoutWnd::MoveItemDown()
{
    QListViewItem* currItem = myLayoutTree->currentItem();
    if( currItem )
    {
	QListViewItem* parent = currItem->parent();
	if( parent )
	{
	    myLayoutTree->setSelected( currItem, FALSE );

	    // Put the items up to, but and including the current item into
	    // the children list.
	    QList<QListViewItem> children;
	    QListViewItem* child;
	    while( (child = parent->firstChild()) != currItem )
	    {
		parent->removeItem( child );
		
		// Reverse the order of the children
		children.insert( 0, child );
	    }

	    QListViewItem* nextChild = currItem->nextSibling();

	    parent->removeItem( child );
	    parent->removeItem( nextChild );

	    // Insert the current, next, then the list of children
	    parent->insertItem( currItem );

	    parent->insertItem( nextChild );
	    
	    QListIterator<QListViewItem> childIter( children );
	    while( (child = childIter()) )
	    {
		parent->insertItem( child );
	    }

	    myLayoutTree->setSelected( currItem, TRUE );

	    emit ContentsChanged();
	    ForceUpdateScreen();
	}
    }
}


void LayoutWnd::MoveItemLeft()
{
    QListViewItem* currItem = myLayoutTree->currentItem();
    if( currItem )
    {
	QListViewItem* parent = currItem->parent();
	if( parent )
	{
	    QListViewItem* grandParent = parent->parent();
	    if( grandParent )
	    {
		myLayoutTree->setSelected( currItem, FALSE );

		// Remove the item from it's current parent
		parent->removeItem( currItem );

		// Put the items up to and including the parent into
		// the children list.
		QList<QListViewItem> children;
		QListViewItem* child;
		while( (child = grandParent->firstChild()) != parent )
		{
		    grandParent->removeItem( child );
		
		    // Reverse the order of the children
		    children.insert( 0, child );
		}

		grandParent->removeItem( parent );
		children.insert( 0, parent );

		// Insert the current then the list of children
		grandParent->insertItem( currItem );
	    
		QListIterator<QListViewItem> childIter( children );
		while( (child = childIter()) )
		{
		    grandParent->insertItem( child );
		}

		myLayoutTree->setSelected( currItem, TRUE );

		emit ContentsChanged();
		ForceUpdateScreen();
	    }
	}
    }
}


void LayoutWnd::MoveItemRight()
{
    QListViewItem* currItem = myLayoutTree->currentItem();
    if( currItem )
    {
	QListViewItem* parent = currItem->parent();
	if( parent )
	{
	    myLayoutTree->setSelected( currItem, FALSE );

	    // Find the previous sibling to the current item
	    QListViewItem* prevSibling = 0;
	    for( prevSibling = parent->firstChild();
		 prevSibling && (prevSibling->nextSibling() != currItem);
		 prevSibling = prevSibling->nextSibling() );
	    
	    if( !prevSibling )
	    {
		return;
	    }
	    
	    parent->removeItem( currItem );

	    // Put all the children of the previous sibling in the list
	    QList<QListViewItem> children;
	    QListViewItem* child;
	    while( (child = prevSibling->firstChild()) )
	    {
		prevSibling->removeItem( child );
		
		// Reverse the order of the children
		children.insert( 0, child );
	    }

	    // Insert the current, then the list of children
	    prevSibling->insertItem( currItem );

	    QListIterator<QListViewItem> childIter( children );
	    while( (child = childIter()) )
	    {
		prevSibling->insertItem( child );
	    }

	    myLayoutTree->setSelected( currItem, TRUE );

	    emit ContentsChanged();
	    ForceUpdateScreen();
	}
    }
}


void LayoutWnd::TreeSelChanged( 
    QListViewItem*	theItem
    )
{
    if( theItem )
    {
        LayoutItem* item = static_cast<LayoutItem*>(theItem);
        if( item->GetType() == LayoutItem::LI_LayoutWidget )
        {
            LayoutWidget* layoutWidget = static_cast<LayoutWidget*>(item);
            const DlgWidget* widget = layoutWidget->GetDlgWidget();
            
            if( widget )
            {
                myLayoutHelper.SelectWidget( widget );
            }
        }
    }

    UpdateScreen();
}


void LayoutWnd::TreeDoubleClicked(
    QListViewItem*      theItem
    )
{
    if( theItem )
    {
        LayoutItem* item = static_cast<LayoutItem*>(theItem);
        if( item->GetType() == LayoutItem::LI_LayoutWidget )
        {
            LayoutWidget* layoutWidget = static_cast<LayoutWidget*>(item);
            const DlgWidget* widget = layoutWidget->GetDlgWidget();
            
            if( widget )
            {
                myLayoutHelper.ShowWidgetProperties( widget );
            }
        }
    }
}


void LayoutWnd::ForceUpdateScreen()
{
    UpdateScreen( TRUE );
}


void LayoutWnd::UpdateScreen(
    bool	forceLayoutRecreate
    )
{
    LayoutItem* currentItem = 
	static_cast<LayoutItem*>(myLayoutTree->currentItem());

    UpdateItemInsertion( currentItem );
    UpdateItemMovement( currentItem );
    
    // We only want to override the forceLayoutRecreate if the update property
    // page indicated a change in the properties
    if( UpdatePropertyPage() )
    {
	forceLayoutRecreate = TRUE;
    }
    
    myLayoutTree->repaint();

    if( forceLayoutRecreate )
    {
	myLayoutHelper.RedrawLayout();
    }

    myPreviousItem = currentItem;
}


void LayoutWnd::Close()
{
    close();
}


bool LayoutWnd::UpdatePropertyPage()
{
    bool propChanged = FALSE;

    LayoutItem* currentItem = 
	static_cast<LayoutItem*>(myLayoutTree->currentItem());

    if( currentItem && currentItem != myPreviousItem )
    {
	LayoutItemPropPage* propPage = 
	    currentItem->GetPropertyPage( myPropertyFrame,
					  currentItem == myRootTreeItem,
					  myLayoutHelper.GetWidgetLayout() );

	propChanged = myPropertyFrame->SetPropertyPage( propPage );
    }

    return propChanged;
}


void LayoutWnd::InsertLayoutItem( 
    LayoutItem::LayoutItemType	newLayoutItem 
    )
{
    LayoutItem* currentItem = 
	static_cast<LayoutItem*>(myLayoutTree->currentItem());
    
    LayoutItem::CreateLayoutItem( newLayoutItem, currentItem );
    
    UpdateScreen( TRUE );
}


LayoutItem* LayoutWnd::InsertRootLayout( 
    LayoutItem::LayoutItemType	newRootItem 
    )
{
    myLayoutTree->clear();
    
    myRootTreeItem = LayoutItem::CreateLayoutItem( newRootItem, myLayoutTree );
    myLayoutTree->setCurrentItem( myRootTreeItem );

    UpdateScreen( TRUE );

    return static_cast<LayoutItem*>(myRootTreeItem);
}


bool LayoutWnd::NoLayoutDefined( void ) const
{
    LayoutItem* currentItem = (LayoutItem*)myLayoutTree->currentItem();

    return currentItem && (currentItem == myRootTreeItem) && 
	(currentItem->GetType() == LayoutItem::LI_NoLayout);
}


void LayoutWnd::UpdateItemInsertion( LayoutItem*	currentItem )
{
    bool canInsertLayout = FALSE;
    bool canInsertBoxItem = FALSE;
    bool canInsertGridItem = FALSE;
    bool canInsertWidget = FALSE;
    bool canInsertGridSpacer = FALSE;

    if( currentItem )
    {
	if( NoLayoutDefined() )
	{
	    canInsertLayout = TRUE;
	}
	else
	{
	    switch( currentItem->GetType() )
	    {
		case LayoutItem::LI_BoxLayout:
		{
		    canInsertLayout = TRUE;
		    canInsertBoxItem = TRUE;
		    canInsertWidget = TRUE;
		    break;
		}
		case LayoutItem::LI_GridLayout:
		{
		    canInsertGridItem = TRUE;
		    break;
		}
		case LayoutItem::LI_GridRow:
		{
		    canInsertLayout = TRUE;
		    canInsertWidget = TRUE;
		    canInsertGridSpacer = TRUE;
		    break;
		}
		case LayoutItem::LI_LayoutWidget:
		{
                    LayoutWidget* layoutWidget = static_cast<LayoutWidget*>
                        (currentItem);
                    
                    if( layoutWidget->WidgetHasOwnLayout() )
                    {
                        canInsertWidget = TRUE;
                    }
                    else
                    {
                        canInsertLayout = (currentItem->childCount() == 0);
                    }
                    
		    break;
		}
		case LayoutItem::LI_NoLayout:
		case LayoutItem::LI_BoxSpacing:
		case LayoutItem::LI_BoxStretch:
		case LayoutItem::LI_GridSpacer:
		default:
		{
		    break;
		}
	    }
	}
    }

    QMenuBar& menu = *menuBar();

    menu.setItemEnabled( myInsertBoxLayoutMenuId, canInsertLayout );
    menu.setItemEnabled( myInsertBoxSpacingMenuId, canInsertBoxItem );
    menu.setItemEnabled( myInsertBoxStretchMenuId, canInsertBoxItem );
    menu.setItemEnabled( myInsertGridLayoutMenuId, canInsertLayout );
    menu.setItemEnabled( myInsertGridRowMenuId, canInsertGridItem );
    menu.setItemEnabled( myInsertGridSpacerMenuId, canInsertGridSpacer );
    menu.setItemEnabled( myInsertLayoutWidgetMenuId, canInsertWidget );

    myInsertBoxLayoutTbId->setEnabled( canInsertLayout );
    myInsertBoxSpacingTbId->setEnabled( canInsertBoxItem );
    myInsertBoxStretchTbId->setEnabled( canInsertBoxItem );
    myInsertGridLayoutTbId->setEnabled( canInsertLayout );
    myInsertGridRowTbId->setEnabled( canInsertGridItem );
    myInsertGridSpacerTbId->setEnabled( canInsertGridSpacer );
    myInsertLayoutWidgetTbId->setEnabled( canInsertWidget );
}


void LayoutWnd::UpdateItemMovement( LayoutItem*		currentItem )
{
    bool canMoveUp = FALSE;
    bool canMoveDown = FALSE;
    bool canMoveLeft = FALSE;
    bool canMoveRight = FALSE;

    if( currentItem && (currentItem != myRootTreeItem) )
    {
	LayoutItem* parentItem = (LayoutItem*)currentItem->parent();

	// Calc canMoveUp
	if( parentItem->firstChild() != currentItem )
	{
	    canMoveUp = TRUE;
	}
	
	// Calc canMoveDown
	if( currentItem->nextSibling() != NULL )
	{
	    canMoveDown = TRUE;
	}

	// Calc canMoveLeft
	if( parentItem != myRootTreeItem )
	{
	    canMoveLeft = 
		CanItemBeChild( currentItem, 
				(LayoutItem*)parentItem->parent() );
	}
	
	// Calc canMoveRight
	if( currentItem != parentItem->firstChild() ) // Not the first child
	{
	    LayoutItem* item = 0;
	    for( item = static_cast<LayoutItem*>(parentItem->firstChild());
		 item->nextSibling() != currentItem;
		 item = static_cast<LayoutItem*>(item->nextSibling()) );
	    
	    canMoveRight = CanItemBeChild( currentItem, item );
	}
    }

    myMoveUpTbId->setEnabled( canMoveUp );
    myMoveDownTbId->setEnabled( canMoveDown );
    myMoveLeftTbId->setEnabled( canMoveLeft );
    myMoveRightTbId->setEnabled( canMoveRight );
    
    myKbdAccel.setItemEnabled( myMoveUpAccelId, canMoveUp );
    myKbdAccel.setItemEnabled( myMoveDownAccelId, canMoveDown );
    myKbdAccel.setItemEnabled( myMoveLeftAccelId, canMoveLeft );
    myKbdAccel.setItemEnabled( myMoveRightAccelId, canMoveRight );
}


bool LayoutWnd::CanItemBeChild( const LayoutItem*       item,
				const LayoutItem*       parent )
{
    bool rc = FALSE;

    LayoutItem::LayoutItemType itemType = item->GetType();
    LayoutItem::LayoutItemType parentType = parent->GetType();

    switch( itemType )
    {
	case LayoutItem::LI_BoxLayout:
	case LayoutItem::LI_GridLayout:
	{
	    rc = ( (parentType == LayoutItem::LI_BoxLayout) ||
		   (parentType == LayoutItem::LI_GridRow) ||
		   ( (parentType == LayoutItem::LI_LayoutWidget) &&
		     (parent->childCount() == 0) ) );
	    break;
	}
	case LayoutItem::LI_LayoutWidget:
	{
            // If the item's immediate parent is a layout widget which has its
            // own layout, the item can not be a child of the given parent.
            const LayoutItem* itemParent = 
                static_cast<const LayoutItem*>(item->parent());
            
            if( itemParent->GetType() == LayoutItem::LI_LayoutWidget )
            {
                const LayoutWidget* parentLayoutWidget = 
                    static_cast<const LayoutWidget*>(itemParent);

                if( parentLayoutWidget->WidgetHasOwnLayout() )
                {
                    break;
                }
            }

            rc = ( 
                (parentType == LayoutItem::LI_BoxLayout) ||
                (parentType == LayoutItem::LI_GridRow)
                );
            
	    break;
	}
	case LayoutItem::LI_BoxSpacing:
	case LayoutItem::LI_BoxStretch:
	{
	    rc = (parentType == LayoutItem::LI_BoxLayout);
	    break;
	}
	case LayoutItem::LI_GridRow:
	{
	    rc = (parentType == LayoutItem::LI_GridLayout);
	    break;
	}
	case LayoutItem::LI_GridSpacer:
	{
	    rc = (parentType == LayoutItem::LI_GridRow);
	    break;
	}
	default:
	{
	    break;
	}
    }

    return rc;
}
