/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKDirectionButton.cpp,v 2.2 1999/10/16 01:45:38 jeff Exp $

*******************************************************************************/


#include "DlgKDirectionButton.h"

#include "KDirectionButtonPropPage.h"

#include "GuiTools.h"

#define Inherited DlgButton

QStrList DlgKDirectionButton::itsSignals;
int DlgKDirectionButton::itsNextWidgetNum = 1;

DlgKDirectionButton::DlgKDirectionButton() :
    Inherited( DlgWidget::DW_Button )
{
}


DlgKDirectionButton::DlgKDirectionButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKDirectionButton::~DlgKDirectionButton()
{
}


void DlgKDirectionButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KDirectionButtonPropPage( this, parent, "KDirectionButton" ) );

    labels.append( "KDirectionButton" );
}


void DlgKDirectionButton::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KDirectionButton {" << endl;
    }
  
    KDirectionButton* widget = GetKDirectionButton();

    stream << saveIndent << "Direction {" 
	   << GuiTools::ArrowTypeStr( widget->direction() )
	   << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKDirectionButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KDirectionButton* widget = GetKDirectionButton();

    // Compare to known keys
    if( key == "Direction" )
    {
	widget->setDirection( GuiTools::ArrowTypeVal( value ) );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKDirectionButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    KDirectionButton* widget = GetKDirectionButton();

    stream << varName << "setDirection( " 
	   << GuiTools::ArrowTypeStr( widget->direction() )
	   << " );" << endl;
}


void DlgKDirectionButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKDirectionButton::GetSignalList()
{
    return itsSignals;
}


void DlgKDirectionButton::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KDirectionButton* widget = 
	new KDirectionButton( theParent,
			      CreateName( "KDirectionButton", 
					  theName, 
					  itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
