/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKIconLoaderButton.cpp,v 2.2 1999/10/16 01:45:43 jeff Exp $

*******************************************************************************/


#include "DlgKIconLoaderButton.h"

#ifndef MODULE_BOOTSTRAP
#include "KIconLoaderButtonPropPage.h"
#endif // MODULE_BOOTSTRAP

#include "GuiTools.h"

#define Inherited DlgPushButton

QStrList DlgKIconLoaderButton::itsSignals;
int DlgKIconLoaderButton::itsNextWidgetNum = 1;

DlgKIconLoaderButton::DlgKIconLoaderButton() :
    Inherited( DlgWidget::DW_PushButton )
{
}


DlgKIconLoaderButton::DlgKIconLoaderButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKIconLoaderButton::~DlgKIconLoaderButton()
{
}


void DlgKIconLoaderButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

#ifndef MODULE_BOOTSTRAP
    pages.append( new KIconLoaderButtonPropPage( this, parent, "KIconLoaderButton" ) );

    labels.append( "KIconLoaderButton" );
#endif // MODULE_BOOTSTRAP
}


void DlgKIconLoaderButton::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KIconLoaderButton {" << endl;
    }
  
    KIconLoaderButton* widget = GetKIconLoaderButton();

    stream << saveIndent << "Icon {" << widget->icon() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKIconLoaderButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KIconLoaderButton* widget = GetKIconLoaderButton();

    // Compare to known keys
    if( key == "Icon" )
    {
	widget->setIcon( value );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKIconLoaderButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    KIconLoaderButton* widget = GetKIconLoaderButton();

    stream << varName << "setIcon( \"" << widget->icon() << "\" );" << endl;
}


void DlgKIconLoaderButton::GenerateCtor(
    QTextStream&	stream,
    const QString&	varName,
    const QString&	parentName
    )
{
    stream << '\t' << varName << " = new " << GetWidgetClass() 
	   << "( " << parentName << " );\n";

    const char* cp = GetWidget()->name();
    if( cp )
    {
	stream << '\t' << varName << "->setName( \"" << cp << "\" );" << endl;
    }
}


void DlgKIconLoaderButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKIconLoaderButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "iconChanged(const char*)" );
    }

    return itsSignals;
}


void DlgKIconLoaderButton::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KIconLoaderButton* widget = 
	new KIconLoaderButton( theParent );
    
    widget->setName( CreateName( "KIconLoaderButton", 
				 theName, 
				 itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
