/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKSpinBox.cpp,v 2.2 1999/10/16 01:46:17 jeff Exp $

*******************************************************************************/


#include "DlgKSpinBox.h"

#include "KSpinBoxPropPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgKSpinBox::itsSignals;
int DlgKSpinBox::itsNextWidgetNum = 1;

DlgKSpinBox::DlgKSpinBox() :
    Inherited( DlgWidget::DW_User )
{
}


DlgKSpinBox::DlgKSpinBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKSpinBox::~DlgKSpinBox()
{
}


void DlgKSpinBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new KSpinBoxPropPage( this, parent, "KSpinBox" ) );

    labels.append( "KSpinBox" );
}


void DlgKSpinBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KSpinBox {" << endl;
    }
  
    KSpinBox* widget = GetKSpinBox();

    stream << saveIndent << "Value {" << widget->getValue() << "}" << endl;
    
    stream << saveIndent << "Editable {";
    SaveBool( stream, widget->isEditable() );
    stream << "}" << endl;
    
    stream << saveIndent << "Alignment {" 
	   << GuiTools::AlignmentStr( widget->getAlign() ) << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKSpinBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KSpinBox* widget = GetKSpinBox();

    // Compare to known keys
    if( key == "Value" )
    {
	widget->setValue( value );
    } 
    else if( key == "Editable" )
    {
	widget->setEditable( (value == "TRUE") );
    }
    else if( key == "Alignment" )
    {
	widget->setAlign( GuiTools::Alignment( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKSpinBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    KSpinBox* widget = GetKSpinBox();

    stream << varName << "KSpinBox::setValue( \"" << widget->getValue() 
	   << "\" );" << endl;
    
    stream << varName << "setEditable( "
	   << ( widget->isEditable() ? "TRUE" : "FALSE" ) << " );" << endl;
    
    stream << varName << "setAlign( "
	   << GuiTools::AlignmentStr( widget->getAlign() ) << " );" << endl;
}


void DlgKSpinBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKSpinBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueIncreased()" );
	itsSignals.append( "valueDecreased()" );
    }

    return itsSignals;
}


void DlgKSpinBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KSpinBox* widget = 
	new KSpinBox( theParent,
		      CreateName( "KSpinBox", 
				  theName, 
				  itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
