/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKTreeList.cpp,v 2.2 1999/10/16 01:46:22 jeff Exp $

*******************************************************************************/


#include "DlgKTreeList.h"

#ifndef MODULE_BOOTSTRAP
#include "KTreeListPropPage.h"
#endif // MODULE_BOOTSTRAP

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgKTreeList::itsSignals;
int DlgKTreeList::itsNextWidgetNum = 1;

DlgKTreeList::DlgKTreeList() :
    Inherited( DlgWidget::DW_Frame )
{
}


DlgKTreeList::DlgKTreeList(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgKTreeList::~DlgKTreeList()
{
}


void DlgKTreeList::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

#ifndef MODULE_BOOTSTRAP
    pages.append( new KTreeListPropPage( this, parent, "KTreeList" ) );

    labels.append( "KTreeList" );
#endif // MODULE_BOOTSTRAP
}


void DlgKTreeList::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "KDE::KTreeList {" << endl;
    }
  
    KTreeList* widget = GetKTreeList();

    stream << saveIndent << "ScrollBar {";
    SaveBool( stream, widget->scrollBar() );
    stream << "}" << endl;

    stream << saveIndent << "BottomScrollBar {";
    SaveBool( stream, widget->bottomScrollBar() );
    stream << "}" << endl;

    stream << saveIndent << "TreeDrawing {";
    SaveBool( stream, widget->treeDrawing() );
    stream << "}" << endl;

    stream << saveIndent << "AutoUpdate {";
    SaveBool( stream, widget->autoUpdate() );
    stream << "}" << endl;

    stream << saveIndent << "SmoothScrolling {";
    SaveBool( stream, widget->smoothScrolling() );
    stream << "}" << endl;

    stream << saveIndent << "IndentSpacing {" 
	   << widget->indentSpacing() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgKTreeList::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    KTreeList* widget = GetKTreeList();

    // Compare to known keys
    if( key == "ScrollBar" )
    {
	widget->setScrollBar( (value == "TRUE" ) );
    } 
    else if( key == "BottomScrollBar" )
    {
	widget->setBottomScrollBar( (value == "TRUE" ) );
    } 
    else if( key == "TreeDrawing" )
    {
	widget->setTreeDrawing( (value == "TRUE" ) );
    } 
    else if( key == "AutoUpdate" )
    {
	widget->setAutoUpdate( (value == "TRUE" ) );
    } 
    else if( key == "SmoothScrolling" )
    {
	widget->setSmoothScrolling( (value == "TRUE" ) );
    } 
    else if( key == "IndentSpacing" )
    {
	widget->setIndentSpacing( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgKTreeList::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const QList<DlgWidget>&	zOrdering
    )
{
    Inherited::GenerateSource( stream, varName, zOrdering );

    KTreeList* widget = GetKTreeList();

    stream << varName << "setScrollBar( " 
	   << (widget->scrollBar() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setBottomScrollBar( " 
	   << (widget->bottomScrollBar() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setTreeDrawing( " 
	   << (widget->treeDrawing() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setAutoUpdate( " 
	   << (widget->autoUpdate() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setSmoothScrolling( " 
	   << (widget->smoothScrolling() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setIndentSpacing( " 
	   << widget->indentSpacing() << " );" << endl;
}


void DlgKTreeList::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgKTreeList::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "collapsed(int)" );
	itsSignals.append( "expanded(int)" );
	itsSignals.append( "highlighted(int)" );
	itsSignals.append( "selected(int)" );
	itsSignals.append( "singleSelected(int)" );
    }

    return itsSignals;
}


void DlgKTreeList::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    KTreeList* widget = 
	new KTreeList( theParent,
		       CreateName( "KTreeList", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
