/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgKURLLabel.h,v 2.2 1999/10/16 01:46:25 jeff Exp $

*******************************************************************************/


#ifndef DlgKURLLabel_included
#define DlgKURLLabel_included

#include <kurllabel.h>
#include <qstring.h>
#include <qstrlist.h>

#include "widget/DlgLabel.h"

class DlgKURLLabel : public DlgLabel
{

public:

    DlgKURLLabel();
    // Constructor

    virtual ~DlgKURLLabel();
    // Destructor
    

    KURLLabel* GetKURLLabel();
    // Get the Qt Widget as KURLLabel.


    const QString& getURL() const;
    
    void setURL(
	const QString&	theURL
	);
    

    bool isGlow() const;
    
    void setGlow(
	bool	isGlow
	);
    

    bool isFloat() const;
    
    void setFloat(
	bool	isFloat
	);
    

    bool isUnderline() const;
    
    void setUnderline(
	bool	isUnderline
	);
    

    TextAlignment getTextAlignment() const;
    
    void setTextAlignment(
	TextAlignment	theTextAlignment
	);
    

    const QColor& getHighlightedColor() const;
    
    void setHighlightedColor(
	const QColor&	theColor
	);
    

    const QColor& getSelectedColor() const;
    
    void setSelectedColor(
	const QColor&	theColor
	);
    

    const QColor& getBackgroundColor() const;
    
    void setBackgroundColor(
	const QColor&	theColor
	);

    //
    // Saving and Loading Methods
    //

    virtual void SaveContents(
	QTextStream&	stream,
	bool		topLevel = TRUE
	);
    // save the contents of the widget

    virtual bool RestoreKeyValue( 
	const QString& 	key, 
	const QString& 	value,
	bool		topLevel = TRUE
	);
    // Restore a single key/value pair


    //
    // Code Generation Methods
    //

    virtual QString GetHeaderFilename() const;
    // return the header filename for the widget

    virtual QString GetWidgetClass() const;
    // return the class name for the widget

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
	const QList<DlgWidget>&	zOrdering
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer.

    //
    // Property Editing Methods
    //

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget


    //
    // Connected Signals List
    //

    virtual void AddSignalsToList( 
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list

    static const QStrList& GetSignalList();
    // method to return the list of signals


protected: // Methods

    DlgKURLLabel(
	DlgWidget::Type	theType
	);
    // Constructor.  Used by a derived class' constructor.

    virtual void CreateQtWidget(
	QWidget*	theParent,
	const char*	theName
	);
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.

private: // attributes

    QString		myURL;

    bool		myIsGlow;
    bool		myIsFloat;
    bool		myIsUnderline;
    
    TextAlignment	myTextAlignment;
    
    QColor		myHighlightedColor;
    QColor		mySelectedColor;
    QColor		myBackgroundColor;

private: // static data

    static QStrList	itsSignals;
    // list of the signals that can be connected to this widget

    static int itsNextWidgetNum;
    // number of the next widget

};


inline QString DlgKURLLabel::GetHeaderFilename() const
{
    return "<kurllabel.h>";
}

inline QString DlgKURLLabel::GetWidgetClass() const
{
    return "KURLLabel";
}


inline KURLLabel* DlgKURLLabel::GetKURLLabel()
{
    return static_cast<KURLLabel*> (GetWidget());
}


inline const QString& DlgKURLLabel::getURL() const
{
    return myURL;
}


inline void DlgKURLLabel::setURL(
    const QString&	theURL
    )
{
    myURL = theURL;
    
    // KURLLabel's url member is just a char*, so set it to be our QString's
    // data so it's valid.
    GetKURLLabel()->setURL( myURL );
}


inline bool DlgKURLLabel::isGlow() const
{
    return myIsGlow;
}


inline void DlgKURLLabel::setGlow(
    bool	isGlow
    )
{
    myIsGlow = isGlow;
    GetKURLLabel()->setGlow( isGlow );
}


inline bool DlgKURLLabel::isFloat() const
{
    return myIsFloat;
}


inline void DlgKURLLabel::setFloat(
    bool	isFloat
    )
{
    myIsFloat = isFloat;
    GetKURLLabel()->setFloat( isFloat );
}


inline bool DlgKURLLabel::isUnderline() const
{
    return myIsUnderline;
}


inline void DlgKURLLabel::setUnderline(
    bool	isUnderline
    )
{
    myIsUnderline = isUnderline;
    GetKURLLabel()->setUnderline( isUnderline );
}


inline TextAlignment DlgKURLLabel::getTextAlignment() const
{
    return myTextAlignment;
}


inline void DlgKURLLabel::setTextAlignment(
    TextAlignment	theTextAlignment
    )
{
    myTextAlignment = theTextAlignment;
    GetKURLLabel()->setTextAlignment( theTextAlignment );
}


inline const QColor& DlgKURLLabel::getHighlightedColor() const
{
    return myHighlightedColor;
}


inline void DlgKURLLabel::setHighlightedColor(
    const QColor&	theColor
    )
{
    myHighlightedColor = theColor;
    GetKURLLabel()->setHighlightedColor( theColor );
}


inline const QColor& DlgKURLLabel::getSelectedColor() const
{
    return mySelectedColor;
}


inline void DlgKURLLabel::setSelectedColor(
    const QColor&	theColor
    )
{
    mySelectedColor = theColor;
    GetKURLLabel()->setSelectedColor( theColor );
}


inline const QColor& DlgKURLLabel::getBackgroundColor() const
{
    return myBackgroundColor;
}


inline void DlgKURLLabel::setBackgroundColor(
    const QColor&	theColor
    )
{
    myBackgroundColor = theColor;
    GetKURLLabel()->setBackgroundColor( theColor );
}



#endif // DlgKURLLabel_included
