/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: test.cpp,v 2.2 1999/10/16 01:47:46 jeff Exp $

*******************************************************************************/


#include "module/WidgetModule.h"

#include "TestPushButton.h"

#include "bmp/PixmapFactory.h"

#include "widget/DlgPushButton.h"

extern "C"
{
    WidgetModule* GetWidgetModule();

    void GetModuleVersion(
        int&    theMajorVersion,
	int&	theMinorVersion
	);
}


class TestWidgetModule : public WidgetModule
{
public:

    TestWidgetModule();
    
    virtual ~TestWidgetModule();

    virtual QString getName();

    virtual bool useToolbar();

    virtual QStrList getWidgetNames();

    virtual QPixmap getWidgetPixmap(
	const QString&	theWidgetName
	);
    
    virtual DlgWidget* newWidget(
	int	theWidgetNum
	);

    virtual DlgWidget* newWidget(
	const QString&	theWidgetName
	);
};


TestWidgetModule::TestWidgetModule()
{
}


TestWidgetModule::~TestWidgetModule()
{
}


QString TestWidgetModule::getName()
{
    return "TEST";
}


bool TestWidgetModule::useToolbar()
{
    return TRUE;
}


QStrList TestWidgetModule::getWidgetNames()
{
    QStrList strList;
    strList.append( "TestPushButton" );
    strList.append( "PushButton" );
    
    return strList;
}


QPixmap TestWidgetModule::getWidgetPixmap(
    const QString&	theWidgetName
    )
{
    QPixmap pixmap;
    
    if( theWidgetName == "TestPushButton" )
    {
	pixmap = PixmapFactory::GetPixmap( PushButton_bmp );
    }
    
    return pixmap;
}

    
DlgWidget* TestWidgetModule::newWidget(
    int	theWidgetNum
    )
{
    DlgWidget* dlgWidget = 0;
    
    switch( theWidgetNum )
    {
	case 0:
	{
	    dlgWidget = new TestPushButton();
	    break;
	}
	case 1:
	{
	    dlgWidget = new DlgPushButton();
	    break;
	}
    }
    
    return dlgWidget;
}


DlgWidget* TestWidgetModule::newWidget(
    const QString&	theModuleName
    )
{
    DlgWidget* dlgWidget = 0;
    
    if( theModuleName == "TestPushButton" )
    {
	dlgWidget = new TestPushButton();
    }
    else if( theModuleName == "PushButton" )
    {
	dlgWidget = new DlgPushButton();
    }
    
    return dlgWidget;
}


WidgetModule* GetWidgetModule()
{
    return new TestWidgetModule();
}


void GetModuleVersion(
    int&        theMajorVersion,
    int&        theMinorVersion
    )
{
    theMajorVersion = 1;
    theMinorVersion = 0;
}

