/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LabelPage.cpp,v 2.4 2000/04/30 17:09:41 jeff Exp $

*******************************************************************************/


#include <qlabel.h>

#include "LabelPage.h"

#include "widget/DlgWidget.h"

#define Inherited LabelPageData

LabelPage::LabelPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    QLabel* label = (QLabel*)widget->GetWidget();
    
    itsText->setText( label->text() );

    switch( label->alignment() & (AlignLeft | AlignHCenter | AlignRight) )
    {
	case AlignLeft:
	default:
	{
	    isHorizLeft->setChecked( TRUE );
	    break;
	}
	case AlignHCenter:
	{
	    isHorizCenter->setChecked( TRUE );
	    break;
	}
	case AlignRight:
	{
	    isHorizRight->setChecked( TRUE );
	    break;
	}
    }
    
    switch( label->alignment() & (AlignTop | AlignVCenter | AlignBottom) )
    {
	case AlignTop:
	{
	    isVertTop->setChecked( TRUE );
	    break;
	}
	case AlignVCenter:
	default:
	{
	    isVertCenter->setChecked( TRUE );
	    break;
	}
	case AlignBottom:
	{
	    isVertBottom->setChecked( TRUE );
	    break;
	}
    }

    switch( label->textFormat() )
    {
        case Qt::PlainText:
        {
            isTextFormatPlain->setChecked( TRUE );
            break;
        }
        case Qt::RichText:
        {
            isTextFormatRich->setChecked( TRUE );
            break;
        }
        case Qt::AutoText:
        {
            isTextFormatAuto->setChecked( TRUE );
            break;
        }
    }
    
    isExpandTabs->setChecked( label->alignment() & ExpandTabs );
    isAutoBreak->setChecked( label->alignment() & WordBreak );

//XXX    itsIndent->setText( QString::number( label->indent() ) );
}


LabelPage::~LabelPage()
{
}


void LabelPage::Apply( DlgWidget*	widget )
{
    QLabel* label = (QLabel*)widget->GetWidget();

    // Set the format before the text
    Qt::TextFormat format = Qt::PlainText;
    if( isTextFormatPlain->isChecked() )
    {
        format = Qt::PlainText;
    }
    else if( isTextFormatRich->isChecked() )
    {
        format = Qt::RichText;
    }
    else if( isTextFormatAuto->isChecked() )
    {
        format = Qt::AutoText;
    }
    label->setTextFormat( format );

    // Force a change in the text
    label->setText( QString() );
    label->setText( itsText->text() );
    
    int alignment = 0;
    
    if( isHorizCenter->isChecked() ) 
    {
	alignment |= AlignHCenter;
    }
    else if( isHorizRight->isChecked() )
    {
	alignment |= AlignRight;
    }
    else // default to left
    {
	alignment |= AlignLeft;
    }
    
    if( isVertTop->isChecked() )
    {
	alignment |= AlignTop;
    }
    else if( isVertBottom->isChecked() )
    {
	alignment |= AlignBottom;
    }
    else // default to center
    {
	alignment |= AlignVCenter;
    }
    
    if( isExpandTabs->isChecked() )
    {
	alignment |= ExpandTabs;
    }
    
    if( isAutoBreak->isChecked() )
    {
	alignment |= WordBreak;
    }
    
    label->setAlignment( alignment );

//XXX    label->setIndent( itsIndent->text().toInt() );
}

