/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ListViewColumnPage.cpp,v 2.2 2000/04/30 17:09:41 jeff Exp $

*******************************************************************************/


/**********************************************************************

	--- Qt Architect generated file ---

	File: ListViewColumnPage.cpp
	Last generated: Sun Aug 23 18:04:56 1998

 *********************************************************************/

#include <qheader.h>
#include <qlistview.h>
#include <qstring.h>

#include "ListViewColumnPage.h"

#include "widget/DlgListView.h"
#include "widget/DlgWidget.h"

#include "GuiTools.h"

#define Inherited ListViewColumnPageData

ListViewColumnPage::ListViewColumnPage
(
    DlgWidget*	widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    const DlgListView* dlgListView = static_cast<DlgListView*>(widget);
    const QListView* listView = static_cast<QListView*>(widget->GetWidget());
    QString str;

    unsigned int i;
    QString widthStr;
    QString alignStr;
    unsigned int numColumns = dlgListView->GetNumHeaders();
    char colNumStr[10];

    for( i = 0; i < numColumns; ++i )
    {
	sprintf( colNumStr, "%d", i + 1 );
	widthStr.setNum( listView->columnWidth( i ) );
	alignStr.setNum( listView->columnAlignment( i ) );

	new QListViewItem( myColumns, 
			   colNumStr,
			   listView->columnText( i ),
			   widthStr,
			   DlgListView::WidthModeToStr( 
			       listView->columnWidthMode( i ) ),
			   alignStr );
			   
    }

    myColumns->setCurrentItem( myColumns->firstChild() );

    updateScreen();
}


ListViewColumnPage::~ListViewColumnPage()
{
}


void ListViewColumnPage::Apply(
    DlgWidget*	theWidget
    )
{
    DlgListView* dlgListView = static_cast<DlgListView*>(theWidget);
    QListView* listView = static_cast<QListView*>(theWidget->GetWidget());
    QString str;

    unsigned int numColumns = GuiTools::numSiblings( myColumns->firstChild() );
    unsigned int numOldColumns = listView->header()->count();

    unsigned int numCurrentColumns = 
	(numColumns < numOldColumns) ? numColumns : numOldColumns;

    // Set the columns which are present
    unsigned int i = 0;
    QListViewItem* item = NULL;
    for( i = 0, item = myColumns->firstChild(); 
	 i < numCurrentColumns; 
	 ++i, item = item->nextSibling() )
    {
	listView->setColumnText( i, item->text( Column_Label ) );

	str = item->text( Column_Width );
	listView->setColumnWidth( i, str.toInt() );
	    
	listView->setColumnWidthMode( 
            i, 
            DlgListView::StrToWidthMode( item->text( Column_WidthMode ) ) 
            );

	str = item->text( Column_Alignment );
	listView->setColumnAlignment( i, str.toInt() );
    }
	
    if( numColumns > numOldColumns )
    {
	// Add new columns
	for( ; i < numColumns; ++i, item = item->nextSibling() )
	{
	    str = item->text( Column_Width );
	    
	    listView->addColumn( item->text( Column_Label ), str.toInt() );
	    listView->setColumnWidthMode( 
		i,
		DlgListView::StrToWidthMode( item->text( Column_WidthMode ) )
		);

	    str = item->text( Column_Alignment );
	    listView->setColumnAlignment( i, str.toInt() );
	}
    }
    else
    {
	// For the extra columns, set the text to the empty text and the
	// width to 0.
	for( ; i < numOldColumns; ++i, item = item->nextSibling() )
	{
	    listView->setColumnText( i, "" );
	    listView->setColumnWidth( i, 0 );
	    listView->setColumnWidthMode( i, QListView::Manual );
	}
    }

    dlgListView->SetNumHeaders( numColumns );

    listView->triggerUpdate();
}


void ListViewColumnPage::updateColumn()
{
    QListViewItem* item = myColumns->currentItem();
    if( item )
    {
	item->setText( Column_Label, myColumnLabel->text() );
	item->setText( Column_Width, myColumnWidth->text() );
	
	QListView::WidthMode mode;
	if( isManualMode->isChecked() )
	{
	    mode = QListView::Manual;
	}
	else if( isMaximumMode->isChecked() )
	{
	    mode = QListView::Maximum;
	}
	
	item->setText( Column_WidthMode, 
		       DlgListView::WidthModeToStr( mode ) );

	int alignment = 0;
	if( isAlignLeft->isChecked() )
	{
	    alignment |= AlignLeft;
	}
	else if( isAlignHCenter->isChecked() )
	{
	    alignment |= AlignHCenter;
	}
	else if( isAlignRight->isChecked() )
	{
	    alignment |= AlignRight;
	}
	
	QString alignStr;
	alignStr.setNum( alignment );
	item->setText( Column_Alignment, alignStr );
    }
}


void ListViewColumnPage::addColumn()
{
    QListView::WidthMode mode;
    if( isManualMode->isChecked() )
    {
	mode = QListView::Manual;
    }
    else if( isMaximumMode->isChecked() )
    {
	mode = QListView::Maximum;
    }
	
    int alignment = 0;
    if( isAlignLeft->isChecked() )
    {
	alignment |= AlignLeft;
    }
    else if( isAlignHCenter->isChecked() )
    {
	alignment |= AlignHCenter;
    }
    else if( isAlignRight->isChecked() )
    {
	alignment |= AlignRight;
    }
	
    QString alignStr;
    alignStr.setNum( alignment );

    char colNumStr[10];
    sprintf( colNumStr, 
	     "%d", 
	     GuiTools::numSiblings( myColumns->firstChild() ) + 1 );

    new QListViewItem( myColumns,
		       colNumStr,
		       myColumnLabel->text(),
		       myColumnWidth->text(),
		       DlgListView::WidthModeToStr( mode ),
		       alignStr );

    updateScreen();
}


void ListViewColumnPage::removeColumn()
{
    delete myColumns->currentItem();
    
    updateScreen();
    renumberColumns();
}


void ListViewColumnPage::columnSelected(
    QListViewItem*	theItem
    )
{
    myColumnLabel->setText( theItem->text( Column_Label ) );
    myColumnWidth->setText( theItem->text( Column_Width ) );
    
    switch( DlgListView::StrToWidthMode( theItem->text( Column_WidthMode ) ) )
    {
	case QListView::Manual:
	{
	    isManualMode->setChecked( TRUE );
	    break;
	}
	case QListView::Maximum:
	{
	    isMaximumMode->setChecked( TRUE );
	    break;
	}
    }

    QString str = theItem->text( Column_Alignment );
    int alignment = str.toInt();
    if( alignment & AlignLeft )
    {
	isAlignLeft->setChecked( TRUE );
    }
    else if( alignment & AlignHCenter )
    {
	isAlignHCenter->setChecked( TRUE );
    }
    else if( alignment & AlignRight )
    {
	isAlignRight->setChecked( TRUE );
    }
}


void ListViewColumnPage::updateScreen()
{
    // Can't have < 1 column
    int numColumns = GuiTools::numSiblings( myColumns->firstChild() );
    myRemoveButton->setEnabled( (numColumns > 1) );
}


void ListViewColumnPage::renumberColumns()
{
    // Renumber all the column numbers
    char colNumStr[10];
    unsigned int i = 1;
    for( QListViewItem* item = myColumns->firstChild();
	 item;
	 item = item->nextSibling(), ++i )
    {
	sprintf( colNumStr, "%d", i );
	item->setText( Column_Number, colNumStr );
    }
}
