/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetPropDialog.cpp,v 2.3 1999/10/16 01:50:13 jeff Exp $

*******************************************************************************/


#include <qstring.h>
#include <qlist.h>
#include <qstrlist.h>

#include "WidgetPropDialog.h"

#include "prop/WidgetVarMapPage.h"
#include "prop/WidgetCommonPage.h"
#include "prop/WidgetColorPage.h"
#include "prop/WidgetGeometryPage.h"

#define Inherited WidgetPropDialogData

WidgetPropDialog::WidgetPropDialog
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name ),
    itsDlgWidget( widget )
{
    QString str = tr( "Widget Properties - %1" ).arg( widget->GetWidgetName() );
    setCaption( str );

    itsVarMapPage = new WidgetVarMapPage( widget, this, "Mapping" );
    itsGeometryPage = new WidgetGeometryPage( widget, this, "Geometry" );
    itsCommonPage = new WidgetCommonPage( widget, this, "Widget" );
    itsColorPage = new WidgetColorPage( widget, this, "Color" );    

    addTab( itsVarMapPage, tr( "Mapping" ) );
    addTab( itsGeometryPage, tr( "Geometry" ) );
    addTab( itsCommonPage, tr( "Widget" ) );
    addTab( itsColorPage, tr( "Colors & Font" ) );

    QList<QWidget> pages;
    QStrList labels;
    
    widget->GetPropertyPages( this, pages, labels );

    QListIterator<QWidget> pageIter( pages );
    QStrListIterator labelIter( labels );
    QWidget* page;
    const char* label;
    
    while( (page = pageIter()) )
    {
	label = labelIter();
	
	addTab( page, label );

	page->connect( this, SIGNAL(Apply(DlgWidget*)), 
		       SLOT(Apply(DlgWidget*)) );
    }

    setCancelButton();
    setApplyButton();

    connect( this, SIGNAL(applyButtonPressed()), SLOT(Apply()) );

    resize( sizeHint() );
}


WidgetPropDialog::~WidgetPropDialog()
{
}


void WidgetPropDialog::Apply()
{
    itsVarMapPage->Apply( itsDlgWidget );
    itsGeometryPage->Apply( itsDlgWidget );
    itsCommonPage->Apply( itsDlgWidget );
    itsColorPage->Apply( itsDlgWidget );

    emit Apply( itsDlgWidget );

    itsDlgWidget->GetWidget()->repaint();
}
