/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgButton.cpp,v 2.8 2000/05/04 03:20:14 jeff Exp $

*******************************************************************************/


#include "widget/DlgButton.h"

#include "prop/ButtonPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgWidget

QStrList DlgButton::itsSignals;
int DlgButton::itsNextWidgetNum = 1;

DlgButton::DlgButton() :
    Inherited( DlgWidget::DW_Button )
{
}


DlgButton::DlgButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgButton::~DlgButton()
{
}


void DlgButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ButtonPage( this, parent, "Button" ) );

    labels.append( tr( "Button" ) );
}

void DlgButton::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "Button" );
  
  QButton* widget = GetButton();

  generator.WriteSingleLine( "Text", widget->text() );
  generator.WriteXMLValue( "AutoRepeat", XMLValue( widget->autoRepeat() ) );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QButton* widget = GetButton();

    // Compare to known keys
    if( key == "Text" )
    {
	widget->setText( value );
    } 
    else if( key == "AutoRepeat" )
    {
	widget->setAutoRepeat( (value == "TRUE") );
    }
    else if( key == "AutoResize" )
    {
        // Deprecated
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgButton::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "Button" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetButton() )
  {
//    QString name = map["Name"].AsQString();
//    UpdateNextWidgetNumber( name, itsNextWidgetNum );

    CreateQtWidget( 0 );
  }
  
  QButton* widget = GetButton();

  widget->setText( map["Text"].AsQString() );
  widget->setAutoRepeat( map["AutoRepeat"].AsBool() );

  // We inherit from 
	Inherited::RestoreXML( map["DlgWidget"] );

}

void DlgButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QButton* widget = GetButton();

    // Don't call setText when the user wants a background pixmap.
    if( !UseBackgroundPixmap() )
    {
	stream << varName << "setText( ";
        GenerateText( widget->text(), widgetLayout, stream );
        stream << " );\n";
    }

    stream << varName << "setAutoRepeat( " << 
	(widget->autoRepeat() ? "TRUE" : "FALSE") << " );\n";
}


void DlgButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "pressed()" );
	itsSignals.append( "released()" );
	itsSignals.append( "clicked()" );
	itsSignals.append( "toggled(bool)" );
        itsSignals.append( "stateChanged(int)" );
    }

    return itsSignals;
}


void DlgButton::CreateQtWidget(
    const char*	theName
    )
{
    QButton* widget = 
	new QButton( GetParent(),
		     CreateName( "Button", 
				 theName, 
				 itsNextWidgetNum ) );

    SetQtWidget( widget );
}
