/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgGroupBox.cpp,v 2.8 2000/05/04 03:20:16 jeff Exp $

*******************************************************************************/


#include "widget/DlgGroupBox.h"

#include "prop/GroupBoxPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgGroupBox::itsSignals;
int DlgGroupBox::itsNextWidgetNum = 1;

DlgGroupBox::DlgGroupBox() :
    Inherited( DlgWidget::DW_GroupBox )
{
}


DlgGroupBox::DlgGroupBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgGroupBox::~DlgGroupBox()
{
}


void DlgGroupBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new GroupBoxPage( this, parent, "GroupBox" ) );

    labels.append( tr( "Group Box" ) );
}

void DlgGroupBox::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "GroupBox" );
  
  QGroupBox* widget = GetGroupBox();

  generator.WriteSingleLine( "Title", widget->title() );

  generator.WriteSingleLine( "Alignment",
                             AlignmentToStr( widget->alignment() ));

  Inherited::Save( generator );

  generator.WriteEndBlock();

}


bool DlgGroupBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QGroupBox* widget = GetGroupBox();

    // Compare to known keys
    if( key == "Title" )
    {
	widget->setTitle( value );
    } 
    else if( key == "Alignment" )
    {
	int alignment = 0;
	
        // Support Left, Right, Center for backwards compatibility
	if( value == "Left" )
	{
	    alignment = AlignLeft;
	}
	else if( value == "Center" )
	{
	    alignment = AlignHCenter;
	}
	else if( value == "Right" )
	{
	    alignment = AlignRight;
	}
        else
        {
            alignment = StrToAlignment( value );
        }
	
	widget->setAlignment( alignment );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgGroupBox::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "GroupBox" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetGroupBox() )
  {
    CreateQtWidget( 0 );
  }
  
  QGroupBox* widget = GetGroupBox();

  widget->setTitle( map["Title"].AsQString() );
  widget->setAlignment( StrToAlignment( map["Alignment"].AsQString() ));
  
  // We inherit from 
	Inherited::RestoreXML( map["Frame"] );

}


void DlgGroupBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QGroupBox* widget = GetGroupBox();

    stream << varName << "setTitle( ";
    GenerateText( widget->title(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << "setAlignment( " << 
        AlignmentToStr( widget->alignment() ) << " );" << endl;
}


void DlgGroupBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgGroupBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgGroupBox::CreateQtWidget(
    const char*	theName
    )
{
    QGroupBox* widget = 
	new QGroupBox( GetParent(),
		       CreateName( "GroupBox", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget );
}
