/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgIconView.cpp,v 2.4 2000/04/30 17:09:47 jeff Exp $

*******************************************************************************/


#include "widget/DlgIconView.h"

#include "prop/IconViewPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgScrollView

QStrList DlgIconView::itsSignals;
int DlgIconView::itsNextWidgetNum = 1;

DlgIconView::DlgIconView() :
    Inherited( DlgWidget::DW_IconView )
{
}


DlgIconView::DlgIconView(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgIconView::~DlgIconView()
{
}


void DlgIconView::AddDemoItems()
{
    QIconView* iconView = GetIconView();
    
    QIconViewItem* item;
    while( (item = iconView->firstItem()) )
    {
        delete item;
    }

    for( int i = 1; i <= 20; ++i )
    {
        (void) new QIconViewItem( iconView, QString( "Item %1" ).arg( i ) );
    }

}


void DlgIconView::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new IconViewPage( this, parent, "IconView" ) );

    labels.append( "IconView" );
}


void DlgIconView::Save(
    QtArchGenerator&    generator
    )
{
    generator.WriteStartBlock( "IconView" );
    
    QIconView* widget = GetIconView();

    generator.WriteXMLValue( "SelectionMode", 
                             SelectionModeToStr( widget->selectionMode() ) );
    generator.WriteXMLValue( "GridX", widget->gridX() );
    generator.WriteXMLValue( "GridY", widget->gridY() );
    generator.WriteXMLValue( "Spacing", widget->spacing() );
    generator.WriteXMLValue( "ItemTextPos", 
                             ItemTextPosToStr( widget->itemTextPos() ) );
    generator.WriteXMLValue( "Arrangement",
                             ArrangementToStr( widget->arrangement() ) );
    generator.WriteXMLValue( "ResizeMode", 
                             ResizeModeToStr( widget->resizeMode() ) );
    generator.WriteXMLValue( "MaxItemWidth", widget->maxItemWidth() );
    generator.WriteXMLValue( "MaxItemTextLength", widget->maxItemTextLength() );
    generator.WriteXMLValue( "AutoArrange", widget->autoArrange() );
    generator.WriteXMLValue( "ShowToolTips", widget->showToolTips() );
    generator.WriteXMLValue( "Sorting", widget->sorting() );
    generator.WriteXMLValue( "SortAscending", widget->sortDirection() );
    generator.WriteXMLValue( "ItemsMovable", widget->itemsMovable() );
    generator.WriteXMLValue( "WordWrapIconText", widget->wordWrapIconText() );

    Inherited::Save( generator );
    
    generator.WriteEndBlock();
}


void DlgIconView::RestoreXML( 
    const XMLValue&   value
    )
{
    // Sanity check
    if( value.GetTypeString() != "IconView" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetIconView() )
    {
        CreateQtWidget( 0 );
    }
  
    QIconView* widget = GetIconView();

    widget->setSelectionMode( 
        StrToSelectionMode( map["SelectionMode"].AsQString() ) );

    widget->setGridX( map["GridX"].AsInt() );
    widget->setGridY( map["GridY"].AsInt() );
    widget->setSpacing( map["Spacing"].AsInt() );
    widget->setItemTextPos( 
        StrToItemTextPos( map["ItemTextPos"].AsQString() ) );
    widget->setArrangement(
        StrToArrangement( map["Arrangement"].AsQString() ) );
    widget->setResizeMode( 
        StrToResizeMode( map["ResizeMode"].AsQString() ) );
    widget->setMaxItemWidth( map["MaxItemWidth"].AsInt() );
    widget->setMaxItemTextLength( map["MaxItemTextLength"].AsInt() );
    widget->setAutoArrange( map["AutoArrange"].AsBool() );
    widget->setShowToolTips( map["ShowToolTips"].AsBool() );
    widget->setSorting( map["Sorting"].AsBool(), map["SortAscending"].AsBool() );
    widget->setItemsMovable( map["ItemsMovable"].AsBool() );
    widget->setWordWrapIconText( map["WordWrapIconText"].AsBool() );

    // We inherit from 
    Inherited::RestoreXML( map["ScrollView"] );
}


void DlgIconView::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QIconView* widget = GetIconView();
    
    stream << varName << "setSelectionMode( QIconView::"
           << SelectionModeToStr( widget->selectionMode() )
           << " );" << endl;
    
    stream << varName << "setGridX( " << widget->gridX() << " );" << endl;
    stream << varName << "setGridY( " << widget->gridY() << " );" << endl;
    
    stream << varName << "setSpacing( " << widget->spacing() << " );" << endl;
    
    stream << varName << "setItemTextPos( QIconView::"
           << ItemTextPosToStr( widget->itemTextPos() )
           << " );" << endl;

    stream << varName << "setArrangement( QIconView::"
           << ArrangementToStr( widget->arrangement() )
           << " );" << endl;
    
    stream << varName << "setResizeMode( QIconView::"
           << ResizeModeToStr( widget->resizeMode() )
           << " );" << endl;
    
    stream << varName << "setMaxItemWidth( " 
           << widget->maxItemWidth() << " );" << endl;
    
    stream << varName << "setMaxItemTextLength( " 
           << widget->maxItemTextLength() << " );" << endl;

    stream << varName << "setAutoArrange( ";
    SaveBool( stream, widget->autoArrange() );
    stream << " );" << endl;
    
    stream << varName << "setShowToolTips( ";
    SaveBool( stream, widget->showToolTips() );
    stream << " );" << endl;
    
    stream << varName << "setSorting( ";
    SaveBool( stream, widget->sorting() );
    stream << ", ";
    SaveBool( stream, widget->sortDirection() );
    stream << " );" << endl;
    
    stream << varName << "setItemsMovable( ";
    SaveBool( stream, widget->itemsMovable() );
    stream << " );" << endl;
    
    stream << varName << "setWordWrapIconText( ";
    SaveBool( stream, widget->wordWrapIconText() );
    stream << " );" << endl;
}


void DlgIconView::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgIconView::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "selectionChanged()" );
	itsSignals.append( "selectionChanged(int)" );
	itsSignals.append( "selectionChanged(QIconViewItem*)" );
	itsSignals.append( "currentChanged()" );
	itsSignals.append( "currentChanged(QIconViewItem*)" );
	itsSignals.append( "clicked(QIconViewItem*)" );
	itsSignals.append( "clicked(QIconViewItem*,const QPoint&)" );
	itsSignals.append( "pressed(QIconViewItem*)" );
	itsSignals.append( "doubleClicked(QIconViewItem*)" );
	itsSignals.append( "returnPressed(QIconViewItem*)" );
	itsSignals.append( "mouseButtonPressed(int,QIconViewItem*,const QPoint&)" );
	itsSignals.append( "mouseButtonClicked(int,QIconViewItem*,const QPoint&)" );
	itsSignals.append( "itemRightClicked(QIconViewItem*)" );
	itsSignals.append( "itemRightPressed(QIconViewItem*)" );
	itsSignals.append( "viewportRightPressed()" );
	itsSignals.append( "viewportRightClicked()" );
	itsSignals.append( "dropped(QDropEvent*)" );
	itsSignals.append( "moved()" );
	itsSignals.append( "onItem(QListViewItem*)" );
	itsSignals.append( "onViewport()" );
	itsSignals.append( "itemRenamed(QIconViewItem*,const QString&)" );
	itsSignals.append( "itemRenamed(QIconViewItem*)" );
    }

    return itsSignals;
}


QString DlgIconView::SelectionModeToStr( QIconView::SelectionMode mode )
{
    QString str;
    
    switch( mode )
    {
        case QIconView::Single:
        {
            str = "Single";
            break;
        }
        case QIconView::Multi:
        {
            str = "Multi";
            break;
        }
        case QIconView::Extended:
        {
            str = "Extended";
            break;
        }
        case QIconView::NoSelection:
        {
            str = "NoSelection";
            break;
        }
    }
    
    return str;
}


QIconView::SelectionMode DlgIconView::StrToSelectionMode( const QString& str )
{
    QIconView::SelectionMode mode = QIconView::Single;
    
    if( str == "Single" )
    {
        mode = QIconView::Single;
    }
    else if( str == "Multi" )
    {
        mode = QIconView::Multi;
    }
    else if( str == "Extended" )
    {
        mode = QIconView::Extended;
    }
    else if( str == "NoSelection" )
    {
        mode = QIconView::NoSelection;
    }
    
    return mode;
}


QString DlgIconView::ArrangementToStr( QIconView::Arrangement arrangement )
{
    QString str;
    
    switch( arrangement )
    {
        case QIconView::LeftToRight:
        {
            str = "LeftToRight";
            break;
        }
        case QIconView::TopToBottom:
        {
            str = "TopToBottom";
            break;
        }
    }
    
    return str;
}


QIconView::Arrangement DlgIconView::StrToArrangement( const QString& str )
{
    QIconView::Arrangement arrangement = QIconView::LeftToRight;
    
    if( str == "LeftToRight" )
    {
        arrangement = QIconView::LeftToRight;
    }
    else if( str == "TopToBottom" )
    {
        arrangement = QIconView::TopToBottom;
    }
    
    return arrangement;
}


QString DlgIconView::ResizeModeToStr( QIconView::ResizeMode  mode )
{
    QString str;
    
    switch( mode )
    {
        case QIconView::Fixed:
        {
            str = "Fixed";
            break;
        }
        case QIconView::Adjust:
        {
            str = "Adjust";
            break;
        }
    }
    
    return str;
}


QIconView::ResizeMode DlgIconView::StrToResizeMode( const QString&      str )
{
    QIconView::ResizeMode mode = QIconView::Fixed;
    
    if( str == "Fixed" )
    {
        mode = QIconView::Fixed;
    }
    else if( str == "Adjust" )
    {
        mode = QIconView::Adjust;
    }
    
    return mode;
}


QString DlgIconView::ItemTextPosToStr( QIconView::ItemTextPos  pos )
{
    QString str;
    
    switch( pos )
    {
        case QIconView::Bottom:
        {
            str = "Bottom";
            break;
        }
        case QIconView::Right:
        {
            str = "Right";
            break;
        }
    }
    
    return str;
}


QIconView::ItemTextPos DlgIconView::StrToItemTextPos( const QString& str )
{
    QIconView::ItemTextPos pos = QIconView::Bottom;
    
    if( str == "Bottom" )
    {
        pos = QIconView::Bottom;
    }
    else if( str == "Right" )
    {
        pos = QIconView::Right;
    }
    
    return pos;
}


void DlgIconView::CreateQtWidget(
    const char*	theName
    )
{
    QIconView* widget = 
	new QIconView( GetParent(),
                       CreateName( "IconView", 
                                   theName, 
                                   itsNextWidgetNum ) );

    SetQtWidget( widget );

    AddDemoItems();
}
