/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgMultiLineEdit.cpp,v 2.8 2000/05/09 02:01:41 jeff Exp $

*******************************************************************************/


#include "widget/DlgMultiLineEdit.h"

#include "prop/MultiLineEditPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgMultiLineEdit::itsSignals;
int DlgMultiLineEdit::itsNextWidgetNum = 1;

DlgMultiLineEdit::DlgMultiLineEdit() :
    Inherited( DlgWidget::DW_MultiLineEdit )
{
}


DlgMultiLineEdit::DlgMultiLineEdit(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgMultiLineEdit::~DlgMultiLineEdit()
{
}


void DlgMultiLineEdit::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new MultiLineEditPage( this, parent, "MultiLineEdit" ) );

    labels.append( tr( "Multi Line Edit" ) );
}

void DlgMultiLineEdit::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "MultiLineEdit" );
  
  QMultiLineEdit* widget = GetMultiLineEdit();

  generator.WriteXMLValue( "Text", widget->text() );

  generator.WriteXMLValue( "Alignment", AlignmentToStr( widget->alignment() ) );
  generator.WriteXMLValue( "EchoMode", EchoModeToStr( widget->echoMode() ) );
  generator.WriteXMLValue( "WordWrap", WordWrapToStr( widget->wordWrap() ) );
  generator.WriteXMLValue( "WrapPolicy", 
                           WrapPolicyToStr( widget->wrapPolicy() ) );
  generator.WriteXMLValue( "WrapColumnOrWidth", widget->wrapColumnOrWidth() );

  generator.WriteXMLValue( "UndoEnabled", widget->isUndoEnabled() );
  generator.WriteXMLValue( "UndoDepth", widget->undoDepth() );
  
  generator.WriteXMLValue( "MaxLength", widget->maxLength() );
  generator.WriteXMLValue( "MaxLineLength", widget->maxLineLength() );
  generator.WriteXMLValue( "MaxLines", widget->maxLines() );
  generator.WriteXMLValue( "HorizontalMargin", widget->hMargin() );
  
  generator.WriteXMLValue( "ReadOnly", widget->isReadOnly() );
  generator.WriteXMLValue( "OverwriteMode", widget->isOverwriteMode() );
  generator.WriteXMLValue( "AutoUpdate", widget->autoUpdate() );
  
  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgMultiLineEdit::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QMultiLineEdit* widget = GetMultiLineEdit();

    // Compare to known keys
    if( key == "Line" )
    {
	widget->insertLine( value );
    } 
    else if( key == "ReadOnly" )
    {
	widget->setReadOnly( (value == "TRUE") );
    }
    else if( key == "OverwriteMode" )
    {
	widget->setOverwriteMode( (value == "TRUE") );
    }
    else if( key == "AutoUpdate" )
    {
	widget->setAutoUpdate( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgMultiLineEdit::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "MultiLineEdit" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetMultiLineEdit() )
  {
    CreateQtWidget( 0 );
  }
  
  QMultiLineEdit* widget = GetMultiLineEdit();

  widget->setText( map["Text"].AsQString() );

  widget->setAlignment( StrToAlignment( map["Alignment"].AsQString() ) );
  widget->setEchoMode( StrToEchoMode( map["EchoMode"].AsQString() ) );
  widget->setWordWrap( StrToWordWrap( map["WordWrap"].AsQString() ) );
  widget->setWrapPolicy( StrToWrapPolicy( map["WrapPolicy"].AsQString() ) );
  widget->setWrapColumnOrWidth( map["WrapColumnOrWidth"].AsInt() );
  
  widget->setUndoEnabled( map["UndoEnabled"].AsBool() );
  widget->setUndoDepth( map["UndoDepth"].AsInt() );
  
  widget->setMaxLength( map["MaxLength"].AsInt() );
  widget->setMaxLineLength( map["MaxLineLength"].AsInt() );
  widget->setMaxLines( map["MaxLines"].AsInt() );
  widget->setHMargin( map["HorizontalMargin"].AsInt() );

  widget->setReadOnly( map["ReadOnly"].AsBool() );
  widget->setOverwriteMode( map["OverwriteMode"].AsBool() );
  widget->setAutoUpdate( map["AutoUpdate"].AsBool() );
  
  // We inherit from 
  Inherited::RestoreXML( map["Frame"] );

}


void DlgMultiLineEdit::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QMultiLineEdit* widget = GetMultiLineEdit();

    stream << varName << "setText( ";
    GenerateText( widget->text(), widgetLayout, stream );
    stream << " );" << endl;

    stream << varName << "setAlignment( "
           << AlignmentToStr( widget->alignment() ) << " );" << endl;
    
    stream << varName << "setEchoMode( QMultiLineEdit::"
           << EchoModeToStr( widget->echoMode() ) << " );" << endl;
    
    stream << varName << "setWordWrap( QMultiLineEdit::"
           << WordWrapToStr( widget->wordWrap() ) << " );" << endl;
    
    stream << varName << "setWrapPolicy( QMultiLineEdit::"
           << WrapPolicyToStr( widget->wrapPolicy() ) << " );" << endl;
    
    stream << varName << "setWrapColumnOrWidth( " 
           << widget->wrapColumnOrWidth() << " );" << endl;
    
    stream << varName << "setUndoEnabled( "
           << (widget->isUndoEnabled() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setUndoDepth( " 
           << widget->undoDepth() << " );" << endl;

    stream << varName << "setMaxLength( "
           << widget->maxLength() << " );" << endl;
    
    stream << varName << "setMaxLineLength( "
           << widget->maxLineLength() << " );" << endl;
    
    stream << varName << "setMaxLines( "
           << widget->maxLines() << " );" << endl;
    
    stream << varName << "setHMargin( "
           << widget->hMargin() << " );" << endl;
    
    stream << varName << "setReadOnly( " <<
	(widget->isReadOnly() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setOverwriteMode( " <<
	(widget->isOverwriteMode() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setAutoUpdate( " <<
	(widget->autoUpdate() ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgMultiLineEdit::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgMultiLineEdit::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "textChanged()" );
	itsSignals.append( "returnPressed()" );
	itsSignals.append( "undoAvailable(bool)" );
	itsSignals.append( "redoAvailable(bool)" );
    }

    return itsSignals;
}


QString DlgMultiLineEdit::EchoModeToStr( QMultiLineEdit::EchoMode echoMode )
{
    QString str;
    
    switch( echoMode )
    {
        case QMultiLineEdit::Normal:
        {
            str = "Normal";
            break;
        }
        case QMultiLineEdit::NoEcho:
        {
            str = "NoEcho";
            break;
        }
        case QMultiLineEdit::Password:
        {
            str = "Password";
            break;
        }
    }
    
    return str;
}


QMultiLineEdit::EchoMode DlgMultiLineEdit::StrToEchoMode( const QString& str )
{       
    QMultiLineEdit::EchoMode echoMode = QMultiLineEdit::Normal;
    
    if( str == "Normal" )
    {
        echoMode = QMultiLineEdit::Normal;
    }
    else if( str == "NoEcho" )
    {
        echoMode = QMultiLineEdit::NoEcho;
    }
    else if( str == "Password" )
    {
        echoMode = QMultiLineEdit::Password;
    }
    
    return echoMode;
}


QString DlgMultiLineEdit::WordWrapToStr( QMultiLineEdit::WordWrap wordWrap )
{
    QString str;
    
    switch( wordWrap )
    {
        case QMultiLineEdit::NoWrap:
        {
            str = "NoWrap";
            break;
        }
        case QMultiLineEdit::WidgetWidth:
        {
            str = "WidgetWidth";
            break;
        }
        case QMultiLineEdit::FixedPixelWidth:
        {
            str = "FixedPixelWidth";
            break;
        }
        case QMultiLineEdit::FixedColumnWidth:
        {
            str = "FixedColumnWidth";
            break;
        }
    }
    
    return str;
}


QMultiLineEdit::WordWrap DlgMultiLineEdit::StrToWordWrap( const QString& str )
{
    QMultiLineEdit::WordWrap wordWrap = QMultiLineEdit::NoWrap;
    
    if( str == "NoWrap" )
    {
        wordWrap = QMultiLineEdit::NoWrap;
    }
    else if( str == "WidgetWidth" )
    {
        wordWrap = QMultiLineEdit::WidgetWidth;
    }
    else if( str == "FixedPixelWidth" )
    {
        wordWrap = QMultiLineEdit::FixedPixelWidth;
    }
    else if( str == "FixedColumnWidth" )
    {
        wordWrap = QMultiLineEdit::FixedColumnWidth;
    }
    
    return wordWrap;
}


QString DlgMultiLineEdit::WrapPolicyToStr( 
    QMultiLineEdit::WrapPolicy wrapPolicy
    )
{
    QString str;
    
    switch( wrapPolicy )
    {
        case QMultiLineEdit::AtWhiteSpace:
        {
            str = "AtWhiteSpace";
            break;
        }
        case QMultiLineEdit::Anywhere:
        {
            str = "Anywhere";
            break;
        }
    }
    
    return str;
}


QMultiLineEdit::WrapPolicy DlgMultiLineEdit::StrToWrapPolicy( 
    const QString& str 
    )
{
    QMultiLineEdit::WrapPolicy wrapPolicy = QMultiLineEdit::AtWhiteSpace;
    
    if( str == "AtWhiteSpace" )
    {
        wrapPolicy = QMultiLineEdit::AtWhiteSpace;
    }
    else if( str == "Anywhere" )
    {
        wrapPolicy = QMultiLineEdit::Anywhere;
    }
    
    return wrapPolicy;
}


void DlgMultiLineEdit::CreateQtWidget(
    const char*	theName
    )
{
    QMultiLineEdit* widget = 
	new QMultiLineEdit( GetParent(),
			    CreateName( "MultiLineEdit", 
					theName, 
					itsNextWidgetNum ) );

    SetQtWidget( widget );
}
