/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgProgressBar.cpp,v 2.8 2000/05/04 03:20:17 jeff Exp $

*******************************************************************************/


#include "widget/DlgProgressBar.h"

#include "prop/ProgressBarPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgProgressBar::itsSignals;
int DlgProgressBar::itsNextWidgetNum = 1;

DlgProgressBar::DlgProgressBar() :
    Inherited( DlgWidget::DW_ProgressBar )
{
}


DlgProgressBar::DlgProgressBar(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgProgressBar::~DlgProgressBar()
{
}


void DlgProgressBar::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ProgressBarPage( this, parent, "ProgressBar" ) );

    labels.append( tr( "Progress Bar" ) );
}

void DlgProgressBar::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "ProgressBar");
  
  QProgressBar* widget = GetProgressBar();

  generator.WriteXMLValue( "TotalSteps", widget->totalSteps() );
  generator.WriteXMLValue( "Progress", widget->progress() );

  generator.WriteXMLValue( "CenterIndicator", widget->centerIndicator() );
  generator.WriteXMLValue( "IndicatorFollowsStyle", 
                           widget->indicatorFollowsStyle() );
    
  Inherited::Save( generator );

  generator.WriteEndBlock();

}

bool DlgProgressBar::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QProgressBar* widget = GetProgressBar();

    // Compare to known keys
    if( key == "TotalSteps" )
    {
	widget->setTotalSteps( value.toInt() );
    } 
    else if( key == "Progress" )
    {
	widget->setProgress( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgProgressBar::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "ProgressBar" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetProgressBar() )
    {
        CreateQtWidget( 0 );
    }
  
    QProgressBar* widget = GetProgressBar();

    widget->setTotalSteps( map["TotalSteps"].AsInt() );
    widget->setProgress( map["Progress"].AsInt() );

    // Must set center indicator first because it sets indicator follows
    // style to false.
    widget->setCenterIndicator( map["CenterIndicator"].AsBool() );
    widget->setIndicatorFollowsStyle( map["IndicatorFollowsStyle"].AsBool() );

    // We inherit from 
    Inherited::RestoreXML( map["Frame"] );

}

void DlgProgressBar::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QProgressBar* widget = GetProgressBar();

    stream << varName << "setProgress( " << widget->progress() << " );" << endl;

    // Must set center indicator first because it sets indicator follows
    // style to false.
    stream << varName << "setCenterIndicator( "
           << (widget->centerIndicator() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setIndicatorFollowsStyle( "
           << (widget->indicatorFollowsStyle() ? "TRUE" : "FALSE") 
           << " );" << endl;
}


void DlgProgressBar::GenerateCtor(
    QTextStream&	stream,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    const QProgressBar* progbar = GetProgressBar();

    stream << progbar->totalSteps() << ", " << parentName << ", " << name;
}


void DlgProgressBar::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgProgressBar::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgProgressBar::CreateQtWidget(
    const char*	theName
    )
{
    QProgressBar* widget = 
	new QProgressBar( GetParent(),
		  CreateName( "ProgressBar", 
			      theName, 
			      itsNextWidgetNum ) );

    SetQtWidget( widget );
}
