/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgRadioButton.cpp,v 2.7 2000/05/04 03:20:17 jeff Exp $

*******************************************************************************/


#include "widget/DlgRadioButton.h"

#include "prop/RadioButtonPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgButton

QStrList DlgRadioButton::itsSignals;
int DlgRadioButton::itsNextWidgetNum = 1;

DlgRadioButton::DlgRadioButton() :
    Inherited( DlgWidget::DW_RadioButton )
{
}


DlgRadioButton::DlgRadioButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgRadioButton::~DlgRadioButton()
{
}


void DlgRadioButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    DlgWidget::GetPropertyPages( parent, pages, labels );

    pages.append( new RadioButtonPage( this, parent, "RadioButton" ) );

    labels.append( tr( "Radio Button" ) );
}

void DlgRadioButton::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "RadioButton" );
  
  QRadioButton* widget = GetRadioButton();

  generator.WriteXMLValue( "Checked", widget->isChecked() );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgRadioButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QRadioButton* widget = GetRadioButton();

    // Compare to known keys
    if( key == "Checked" )
    {
	widget->setChecked( (value == "TRUE") );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgRadioButton::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "RadioButton" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetRadioButton() )
  {
    CreateQtWidget( 0 );
  }
  
  QRadioButton* widget = GetRadioButton();

  widget->setChecked( map["Checked"].AsBool() );
  
  // We inherit from 
	Inherited::RestoreXML( map["Button"] );

}


void DlgRadioButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QRadioButton* widget = GetRadioButton();

    stream << varName << "setChecked( " <<
	(widget->isChecked() ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgRadioButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgRadioButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgRadioButton::CreateQtWidget(
    const char*	theName
    )
{
    QRadioButton* widget = 
	new QRadioButton( GetParent(),
			  CreateName( "RadioButton", 
				      theName, 
				      itsNextWidgetNum ) );

    SetQtWidget( widget );
}
