/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgUser.cpp,v 2.6 2000/04/29 20:15:55 klaus Exp $

*******************************************************************************/


#include <qlabel.h>

#include "widget/DlgUser.h"

#include "prop/UserPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgWidget

QStrList DlgUser::itsSignals;
int DlgUser::itsNextWidgetNum = 1;

DlgUser::DlgUser() :
    Inherited( DlgWidget::DW_User )
{
}


DlgUser::DlgUser(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgUser::~DlgUser()
{
}


void DlgUser::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new UserPage( this, parent, "User" ) );

    labels.append( tr( "User Widget" ) );
}

void DlgUser::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "User" );

  generator.WriteSingleLine( "UserClassHeader", myUserClassHeader );
  generator.WriteSingleLine( "UserClassName", myUserClassName );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgUser::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "UserClassHeader" )
    {
	myUserClassHeader = value;
    }
    else if( key == "UserClassName" )
    {
	myUserClassName = value;
	static_cast<QLabel*>(GetWidget())->setText( value );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgUser::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "User" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetWidget() )
  {
    CreateQtWidget( 0 );
  }
  
  QLabel* widget = static_cast<QLabel*>( GetWidget() );

	myUserClassHeader = map["UserClassHeader"].AsQString();
	myUserClassName = map["UserClassName"].AsQString();
	widget->setText( myUserClassName );

  // We inherit from 
	Inherited::RestoreXML( map["DlgWidget"] );

}


void DlgUser::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );
}


void DlgUser::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgUser::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgUser::CreateQtWidget(
    const char*	theName
    )
{
    QLabel* widget = new QLabel( GetParent(),
				 CreateName( "User",
					     theName,
					     itsNextWidgetNum ) );
    
    widget->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
    widget->setAlignment( AlignCenter );
    
    SetQtWidget( widget );
}
