/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgUser.h,v 2.5 2000/04/29 20:15:55 klaus Exp $

*******************************************************************************/


#ifndef DlgUser_included
#define DlgUser_included

#include <qstring.h>
#include <qstrlist.h>

#include "widget/DlgWidget.h"

class DlgUser : public DlgWidget
{

public:

    DlgUser();
    // Constructor

    virtual ~DlgUser();
    // Destructor
    

    //
    // Saving and Loading Methods
    //

    virtual void Save( QtArchGenerator & );
    // save the contents of the widget

    virtual bool RestoreKeyValue( 
	const QString& 	key, 
	const QString& 	value,
	bool		topLevel = TRUE
	);
    // Restore a single key/value pair

    virtual void RestoreXML( const XMLValue & );
    // Restore a XMLValue


    //
    // Code Generation Methods
    //

    virtual QString GetHeaderFilename() const;
    // return the header filename for the widget

    virtual QString GetWidgetClass() const;
    // return the class name for the widget

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
        const WidgetLayout&     widgetLayout
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer.


    //
    // Property Editing Methods
    //

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget


    //
    // Connected Signals List
    //

    virtual void AddSignalsToList( 
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list

    static const QStrList& GetSignalList();
    // method to return the list of signals


    //
    // User widget properties
    //

    void SetUserClassHeader(
	const QString&	theHeader
	);
    // Set the class header.

    const QString& GetUserClassHeader() const;
    // Get the class header.

    void SetUserClassName(
	const QString&	theName
	);
    // Set the class name

    // GetWidgetClass will return the user class name.


protected: // Methods

    DlgUser(
	DlgWidget::Type	theType
	);
    // Constructor.  Used by a derived class' constructor.

    virtual void CreateQtWidget(
	const char*	theName
	);
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.


private: // Methods


private: // static data

    static QStrList	itsSignals;
    // list of the signals that can be connected to this widget

    static int itsNextWidgetNum;
    // number of the next widget

private: // Attributes

    QString	myUserClassHeader;
    // Filename containing the class declaration for the widget

    QString	myUserClassName;
    // Name of the user class
};


inline QString DlgUser::GetHeaderFilename() const
{
    return "\"" + myUserClassHeader + "\"";
}

inline QString DlgUser::GetWidgetClass() const
{
    return myUserClassName;
}


inline void DlgUser::SetUserClassHeader(
    const QString&	theHeader
    )
{
    myUserClassHeader = theHeader;
}


inline const QString& DlgUser::GetUserClassHeader() const
{
    return myUserClassHeader;
}


inline void DlgUser::SetUserClassName(
    const QString&	theName
    )
{
    myUserClassName = theName;
}


#endif // DlgUser_included
